@@ -1,6 +1,6 @@
 // * This makes emacs happy -*-Mode: C++;-*-
 /****************************************************************************
- * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2005,2007 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -31,13 +31,15 @@
  *   Author: Juergen Pfeifer, 1997                                          *
  ****************************************************************************/
 
-// $Id: internal.h,v 1.10 2003/10/25 15:04:46 tom Exp $
+// $Id: internal.h,v 1.15 2007/06/02 15:58:00 tom Exp $
 
 #ifndef NCURSES_CPLUS_INTERNAL_H
 #define NCURSES_CPLUS_INTERNAL_H 1
 
 #include <ncurses_cfg.h>
 
+#include <stdlib.h>
+
 #if USE_RCS_IDS
 #define MODULE_ID(id) static const char Ident[] = id;
 #else
@@ -46,4 +48,8 @@
 
 #define CTRL(x) ((x) & 0x1f)
 
-#endif // NCURSES_CPLUS_INTERNAL_H
+#ifndef _QNX_SOURCE
+#include <string.h>
+#endif
+
+#endif /* NCURSES_CPLUS_INTERNAL_H */
