@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2004,2005 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -32,7 +32,7 @@
 
 #include "form.priv.h"
 
-MODULE_ID("$Id: frm_opts.c,v 1.11 2003/10/25 15:17:08 tom Exp $")
+MODULE_ID("$Id: frm_opts.c,v 1.14 2005/04/16 16:59:18 tom Exp $")
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnform  
@@ -45,14 +45,16 @@ MODULE_ID("$Id: frm_opts.c,v 1.11 2003/10/25 15:17:08 tom Exp $")
 |                    E_BAD_ARGUMENT    - invalid options
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-set_form_opts (FORM * form, Form_Options  opts)
+set_form_opts(FORM *form, Form_Options opts)
 {
+  T((T_CALLED("set_form_opts(%p,%d)"), form, opts));
+
   opts &= ALL_FORM_OPTS;
   if (opts & ~ALL_FORM_OPTS)
     RETURN(E_BAD_ARGUMENT);
   else
     {
-      Normalize_Form( form )->opts = opts;
+      Normalize_Form(form)->opts = opts;
       RETURN(E_OK);
     }
 }
@@ -66,9 +68,10 @@ set_form_opts (FORM * form, Form_Options  opts)
 |   Return Values :  The option flags.
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(Form_Options)
-form_opts (const FORM * form)
+form_opts(const FORM *form)
 {
-  return (Normalize_Form(form)->opts & ALL_FORM_OPTS);
+  T((T_CALLED("form_opts(%p)"), form));
+  returnCode((int)(Normalize_Form(form)->opts & ALL_FORM_OPTS));
 }
 
 /*---------------------------------------------------------------------------
@@ -82,14 +85,16 @@ form_opts (const FORM * form)
 |                    E_BAD_ARGUMENT  - invalid options
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-form_opts_on (FORM * form, Form_Options opts)
+form_opts_on(FORM *form, Form_Options opts)
 {
+  T((T_CALLED("form_opts_on(%p,%d)"), form, opts));
+
   opts &= ALL_FORM_OPTS;
   if (opts & ~ALL_FORM_OPTS)
     RETURN(E_BAD_ARGUMENT);
   else
     {
-      Normalize_Form( form )->opts |= opts;	
+      Normalize_Form(form)->opts |= opts;
       RETURN(E_OK);
     }
 }
@@ -105,8 +110,10 @@ form_opts_on (FORM * form, Form_Options opts)
 |                    E_BAD_ARGUMENT  - invalid options
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-form_opts_off (FORM * form, Form_Options opts)
+form_opts_off(FORM *form, Form_Options opts)
 {
+  T((T_CALLED("form_opts_off(%p,%d)"), form, opts));
+
   opts &= ALL_FORM_OPTS;
   if (opts & ~ALL_FORM_OPTS)
     RETURN(E_BAD_ARGUMENT);
