@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2003,2004 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -32,7 +32,7 @@
 
 #include "form.priv.h"
 
-MODULE_ID("$Id: frm_win.c,v 1.11 2003/10/25 15:17:08 tom Exp $")
+MODULE_ID("$Id: frm_win.c,v 1.13 2004/12/11 22:15:27 tom Exp $")
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnform  
@@ -44,14 +44,16 @@ MODULE_ID("$Id: frm_win.c,v 1.11 2003/10/25 15:17:08 tom Exp $")
 |                    E_POSTED   - form is posted
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-set_form_win (FORM * form, WINDOW * win)
+set_form_win(FORM *form, WINDOW *win)
 {
-  if (form && (form->status & _POSTED))	
+  T((T_CALLED("set_form_win(%p,%p)"), form, win));
+
+  if (form && (form->status & _POSTED))
     RETURN(E_POSTED);
 
-  Normalize_Form( form )->win = win;
+  Normalize_Form(form)->win = win;
   RETURN(E_OK);
-}	
+}
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnform  
@@ -62,11 +64,14 @@ set_form_win (FORM * form, WINDOW * win)
 |   Return Values :  The pointer to the Window or stdscr if there is none.
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(WINDOW *)
-form_win (const FORM * form)
+form_win(const FORM *form)
 {
-  const FORM* f = Normalize_Form( form );
-  return (f->win ? f->win : stdscr);
+  const FORM *f;
+
+  T((T_CALLED("form_win(%p)"), form));
+
+  f = Normalize_Form(form);
+  returnWin(f->win ? f->win : stdscr);
 }
 
 /* frm_win.c ends here */
-
