@@ -2,10 +2,10 @@
 #
 # MKlib_gen.sh -- generate sources from curses.h macro definitions
 #
-# ($Id: MKlib_gen.sh,v 1.24 2005/06/11 19:26:05 tom Exp $)
+# ($Id: MKlib_gen.sh,v 1.29 2007/08/18 13:17:14 tom Exp $)
 #
 ##############################################################################
-# Copyright (c) 1998-2004,2005 Free Software Foundation, Inc.                #
+# Copyright (c) 1998-2006,2007 Free Software Foundation, Inc.                #
 #                                                                            #
 # Permission is hereby granted, free of charge, to any person obtaining a    #
 # copy of this software and associated documentation files (the "Software"), #
@@ -62,7 +62,7 @@ if test "${LC_MESSAGES+set}" = set; then LC_MESSAGES=C; export LC_MESSAGES; fi
 if test "${LC_CTYPE+set}"    = set; then LC_CTYPE=C;    export LC_CTYPE;    fi
 if test "${LC_COLLATE+set}"  = set; then LC_COLLATE=C;  export LC_COLLATE;  fi
 
-preprocessor="$1 -I../include"
+preprocessor="$1 -DNCURSES_INTERNALS -I../include"
 AWK="$2"
 USE="$3"
 
@@ -380,6 +380,7 @@ BEGIN		{
 			print " * pull most of the rest of the library into your link image."
 		}
 		print " */"
+		print "#define NCURSES_ATTR_T int"
 		print "#include <curses.priv.h>"
 		print ""
 		}
@@ -394,6 +395,7 @@ EOF1
 
 cat >$TMP <<EOF
 #include <ncurses_cfg.h>
+#undef NCURSES_NOMACROS
 #include <curses.h>
 
 DECLARATIONS
@@ -404,10 +406,17 @@ sed -n -f $ED1 \
 | sed -e 's/NCURSES_EXPORT(\(.*\)) \(.*\) (\(.*\))/\1 \2(\3)/' \
 | sed -f $ED2 \
 | $AWK -f $AW1 using=$USE \
-| sed -e 's/^\([a-z_][a-z_]*[ *]*\)/\1 gen_/' -e 's/  / /g' >>$TMP
+| sed \
+	-e 's/ [ ]*$//g' \
+	-e 's/^\([a-zA-Z_][a-zA-Z_]*[ *]*\)/\1 gen_/' \
+	-e 's/gen_$//' \
+	-e 's/  / /g' >>$TMP
 
 $preprocessor $TMP 2>/dev/null \
-| sed -e 's/  / /g' -e 's/^ //' \
+| sed \
+	-e 's/  / /g' \
+	-e 's/^ //' \
+	-e 's/^_Bool/bool/' \
 | $AWK -f $AW2 \
 | sed -f $ED3 \
 | sed \
