@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2006,2007 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -30,7 +30,7 @@
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
  *                                                                          *
- *  Rewritten 2001-2002 to support wide-characters by                       *
+ *  Rewritten 2001-2004 to support wide-characters by                       *
  *	Sven Verdoolaege                                                    *
  *	Thomas Dickey                                                       *
  ****************************************************************************/
@@ -44,7 +44,7 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: lib_addstr.c,v 1.39 2003/12/06 18:04:33 tom Exp $")
+MODULE_ID("$Id: lib_addstr.c,v 1.47 2007/04/07 17:13:21 tom Exp $")
 
 NCURSES_EXPORT(int)
 waddnstr(WINDOW *win, const char *astr, int n)
@@ -55,7 +55,8 @@ waddnstr(WINDOW *win, const char *astr, int n)
     T((T_CALLED("waddnstr(%p,%s,%d)"), win, _nc_visbufn(astr, n), n));
 
     if (win && (str != 0)) {
-	TR(TRACE_VIRTPUT | TRACE_ATTRS, ("... current %s", _traceattr(win->_attrs)));
+	TR(TRACE_VIRTPUT | TRACE_ATTRS,
+	   ("... current %s", _traceattr(WINDOW_ATTRS(win))));
 	code = OK;
 	if (n < 0)
 	    n = (int) strlen(astr);
@@ -79,9 +80,9 @@ waddnstr(WINDOW *win, const char *astr, int n)
 NCURSES_EXPORT(int)
 waddchnstr(WINDOW *win, const chtype *astr, int n)
 {
-    NCURSES_SIZE_T y = win->_cury;
-    NCURSES_SIZE_T x = win->_curx;
+    NCURSES_SIZE_T y, x;
     int code = OK;
+    int i;
     struct ldat *line;
 
     T((T_CALLED("waddchnstr(%p,%p,%d)"), win, astr, n));
@@ -89,6 +90,8 @@ waddchnstr(WINDOW *win, const chtype *astr, int n)
     if (!win)
 	returnCode(ERR);
 
+    y = win->_cury;
+    x = win->_curx;
     if (n < 0) {
 	const chtype *str;
 	n = 0;
@@ -101,15 +104,9 @@ waddchnstr(WINDOW *win, const chtype *astr, int n)
 	returnCode(code);
 
     line = &(win->_line[y]);
-#if USE_WIDEC_SUPPORT
-    {
-	int i;
-	for (i = 0; i < n; ++i)
-	    SetChar(line->text[i + x], ChCharOf(astr[i]), ChAttrOf(astr[i]));
+    for (i = 0; i < n && ChCharOf(astr[i]) != '\0'; ++i) {
+	SetChar2(line->text[i + x], astr[i]);
     }
-#else
-    memcpy(line->text + x, astr, n * sizeof(*astr));
-#endif
     CHANGED_RANGE(line, x, x + n - 1);
 
     _nc_synchook(win);
@@ -119,7 +116,7 @@ waddchnstr(WINDOW *win, const chtype *astr, int n)
 #if USE_WIDEC_SUPPORT
 
 NCURSES_EXPORT(int)
-_nc_wchstrlen(const cchar_t * s)
+_nc_wchstrlen(const cchar_t *s)
 {
     int result = 0;
     while (CharOf(s[result]) != L'\0') {
@@ -129,13 +126,14 @@ _nc_wchstrlen(const cchar_t * s)
 }
 
 NCURSES_EXPORT(int)
-wadd_wchnstr(WINDOW *win, const cchar_t * astr, int n)
+wadd_wchnstr(WINDOW *win, const cchar_t *astr, int n)
 {
+    static const NCURSES_CH_T blank = NewChar(BLANK_TEXT);
     NCURSES_SIZE_T y = win->_cury;
     NCURSES_SIZE_T x = win->_curx;
     int code = OK;
     struct ldat *line;
-    int i, start, end;
+    int i, j, start, len, end;
 
     T((T_CALLED("wadd_wchnstr(%p,%s,%d)"), win, _nc_viscbuf(astr, n), n));
 
@@ -153,22 +151,58 @@ wadd_wchnstr(WINDOW *win, const cchar_t * astr, int n)
     line = &(win->_line[y]);
     start = x;
     end = x + n - 1;
-    if (isnac(line->text[x])) {
-	line->text[x - 1] = win->_nc_bkgd;
-	--start;
-    }
-    for (i = 0; i < n && x <= win->_maxx; ++i) {
-	line->text[x++] = astr[i];
-	if (wcwidth(CharOf(astr[i])) > 1) {
-	    if (x <= win->_maxx)
-		AddAttr(line->text[x++], WA_NAC);
-	    else
-		line->text[x - 1] = win->_nc_bkgd;
+
+    /*
+     * Reset orphaned cells of multi-column characters that extend up to the
+     * new string's location to blanks.
+     */
+    if (x > 0 && isWidecExt(line->text[x])) {
+	for (i = 0; i <= x; ++i) {
+	    if (!isWidecExt(line->text[x - i])) {
+		/* must be isWidecBase() */
+		start -= i;
+		while (i > 0) {
+		    line->text[x - i--] = _nc_render(win, blank);
+		}
+		break;
+	    }
 	}
     }
-    if (x <= win->_maxx && isnac(line->text[x])) {
-	line->text[x] = win->_nc_bkgd;
+
+    /*
+     * Copy the new string to the window.
+     */
+    for (i = 0; i < n && CharOf(astr[i]) != L'\0' && x <= win->_maxx; ++i) {
+	if (isWidecExt(astr[i]))
+	    continue;
+
+	len = wcwidth(CharOf(astr[i]));
+
+	if (x + len - 1 <= win->_maxx) {
+	    line->text[x] = _nc_render(win, astr[i]);
+	    if (len > 1) {
+		for (j = 0; j < len; ++j) {
+		    if (j != 0) {
+			line->text[x + j] = line->text[x];
+		    }
+		    SetWidecExt(line->text[x + j], j);
+		}
+	    }
+	    x += len;
+	    end += len - 1;
+	} else {
+	    break;
+	}
+    }
+
+    /*
+     * Set orphaned cells of multi-column characters which lie after the new
+     * string to blanks.
+     */
+    while (x <= win->_maxx && isWidecExt(line->text[x])) {
+	line->text[x] = _nc_render(win, blank);
 	++end;
+	++x;
     }
     CHANGED_RANGE(line, start, end);
 
@@ -177,14 +211,15 @@ wadd_wchnstr(WINDOW *win, const cchar_t * astr, int n)
 }
 
 NCURSES_EXPORT(int)
-waddnwstr(WINDOW *win, const wchar_t * str, int n)
+waddnwstr(WINDOW *win, const wchar_t *str, int n)
 {
     int code = ERR;
 
     T((T_CALLED("waddnwstr(%p,%s,%d)"), win, _nc_viswbufn(str, n), n));
 
     if (win && (str != 0)) {
-	TR(TRACE_VIRTPUT | TRACE_ATTRS, ("... current %s", _traceattr(win->_attrs)));
+	TR(TRACE_VIRTPUT | TRACE_ATTRS,
+	   ("... current %s", _traceattr(WINDOW_ATTRS(win))));
 	code = OK;
 	if (n < 0)
 	    n = (int) wcslen(str);
