@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2004,2005 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -27,30 +27,30 @@
  ****************************************************************************/
 
 /****************************************************************************
- *  Author: Thomas E. Dickey <dickey@clark.net> 1997,1999                   *
+ *  Author: Thomas E. Dickey                                                *
  ****************************************************************************/
 
 #include <curses.priv.h>
 #include <term.h>
 
-MODULE_ID("$Id: lib_dft_fgbg.c,v 1.13 2000/07/07 16:50:27 tom Exp $")
+MODULE_ID("$Id: lib_dft_fgbg.c,v 1.18 2005/11/26 20:03:38 tom Exp $")
 
 /*
  * Modify the behavior of color-pair 0 so that the library doesn't assume that
  * it is white on black.  This is an extension to XSI curses.
  */
-int
+NCURSES_EXPORT(int)
 use_default_colors(void)
 {
     T((T_CALLED("use_default_colors()")));
-    returnCode(assume_default_colors(C_MASK, C_MASK));
+    returnCode(assume_default_colors(-1, -1));
 }
 
 /*
  * Modify the behavior of color-pair 0 so that the library assumes that it
  * is something specific, possibly not white on black.
  */
-int
+NCURSES_EXPORT(int)
 assume_default_colors(int fg, int bg)
 {
     T((T_CALLED("assume_default_colors(%d,%d)"), fg, bg));
@@ -61,11 +61,15 @@ assume_default_colors(int fg, int bg)
     if (initialize_pair)	/* don't know how to handle this */
 	returnCode(ERR);
 
-    SP->_default_color = (fg < 0 || fg == C_MASK) || (bg < 0 || bg == C_MASK);
+    SP->_default_color = isDefaultColor(fg) || isDefaultColor(bg);
     SP->_has_sgr_39_49 = (tigetflag("AX") == TRUE);
-    SP->_default_fg = (fg >= 0) ? (fg & C_MASK) : C_MASK;
-    SP->_default_bg = (bg >= 0) ? (bg & C_MASK) : C_MASK;
-    if (SP->_color_pairs != 0)
-	init_pair(0, fg, bg);
+    SP->_default_fg = isDefaultColor(fg) ? COLOR_DEFAULT : (fg & C_MASK);
+    SP->_default_bg = isDefaultColor(bg) ? COLOR_DEFAULT : (bg & C_MASK);
+    if (SP->_color_pairs != 0) {
+	bool save = SP->_default_color;
+	SP->_default_color = TRUE;
+	init_pair(0, (short) fg, (short) bg);
+	SP->_default_color = save;
+    }
     returnCode(OK);
 }
