@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2005,2006 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,15 +29,18 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
- *     and: Thomas E. Dickey 1996-2003                                      *
+ *     and: Thomas E. Dickey                        1996-on                 *
  ****************************************************************************/
 
 /*
  * This module is intended to encapsulate ncurses's interface to pointing
  * devices.
  *
- * The first method used is xterm's internal mouse-tracking facility.
- * The second is Alessandro Rubini's GPM server.
+ * The primary method used is xterm's internal mouse-tracking facility.
+ * Additional methods depend on the platform:
+ *	Alessandro Rubini's GPM server (Linux)
+ *	sysmouse (FreeBSD)
+ *	special-purpose mouse interface for OS/2 EMX.
  *
  * Notes for implementors of new mouse-interface methods:
  *
@@ -76,18 +79,30 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: lib_mouse.c,v 1.68 2003/11/08 21:50:50 tom Exp $")
+MODULE_ID("$Id: lib_mouse.c,v 1.87 2006/12/30 16:30:06 tom Exp $")
 
 #include <term.h>
 #include <tic.h>
 
 #if USE_GPM_SUPPORT
-#ifndef LINT			/* don't need this for llib-lncurses */
-#undef buttons			/* term.h defines this, and gpm uses it! */
-#include <gpm.h>
 #include <linux/keyboard.h>	/* defines KG_* macros */
+
+#ifdef HAVE_LIBDL
+/* use dynamic loader to avoid linkage dependency */
+#include <dlfcn.h>
+
+#ifdef RTLD_NOW
+#define my_RTLD RTLD_NOW
+#else
+#ifdef RTLD_LAZY
+#define my_RTLD RTLD_LAZY
+#else
+make an error
 #endif
-#endif
+#endif				/* RTLD_NOW */
+#endif				/* HAVE_LIBDL */
+
+#endif				/* USE_GPM_SUPPORT */
 
 #if USE_SYSMOUSE
 #undef buttons			/* symbol conflict in consio.h */
@@ -99,31 +114,45 @@ MODULE_ID("$Id: lib_mouse.c,v 1.68 2003/11/08 21:50:50 tom Exp $")
 #else
 #include <machine/console.h>
 #endif
-#endif /* use_SYSMOUSE */
+#endif				/* use_SYSMOUSE */
 
 #define MY_TRACE TRACE_ICALLS|TRACE_IEVENT
 
-#define	MASK_RELEASE(x)		((001 << (6 * ((x) - 1))))
-#define	MASK_PRESS(x)		((002 << (6 * ((x) - 1))))
-#define	MASK_CLICK(x)		((004 << (6 * ((x) - 1))))
-#define	MASK_DOUBLE_CLICK(x)	((010 << (6 * ((x) - 1))))
-#define	MASK_TRIPLE_CLICK(x)	((020 << (6 * ((x) - 1))))
-#define	MASK_RESERVED_EVENT(x)	((040 << (6 * ((x) - 1))))
+#define	MASK_RELEASE(x)		NCURSES_MOUSE_MASK(x, 001)
+#define	MASK_PRESS(x)		NCURSES_MOUSE_MASK(x, 002)
+#define	MASK_CLICK(x)		NCURSES_MOUSE_MASK(x, 004)
+#define	MASK_DOUBLE_CLICK(x)	NCURSES_MOUSE_MASK(x, 010)
+#define	MASK_TRIPLE_CLICK(x)	NCURSES_MOUSE_MASK(x, 020)
+#define	MASK_RESERVED_EVENT(x)	NCURSES_MOUSE_MASK(x, 040)
 
-#define BUTTON_CLICKED  (BUTTON1_CLICKED  | BUTTON2_CLICKED  | BUTTON3_CLICKED)
-#define BUTTON_PRESSED  (BUTTON1_PRESSED  | BUTTON2_PRESSED  | BUTTON3_PRESSED)
-#define BUTTON_RELEASED (BUTTON1_RELEASED | BUTTON2_RELEASED | BUTTON3_RELEASED)
+#if NCURSES_MOUSE_VERSION == 1
+#define BUTTON_CLICKED        (BUTTON1_CLICKED        | BUTTON2_CLICKED        | BUTTON3_CLICKED        | BUTTON4_CLICKED)
+#define BUTTON_PRESSED        (BUTTON1_PRESSED        | BUTTON2_PRESSED        | BUTTON3_PRESSED        | BUTTON4_PRESSED)
+#define BUTTON_RELEASED       (BUTTON1_RELEASED       | BUTTON2_RELEASED       | BUTTON3_RELEASED       | BUTTON4_RELEASED)
+#define BUTTON_DOUBLE_CLICKED (BUTTON1_DOUBLE_CLICKED | BUTTON2_DOUBLE_CLICKED | BUTTON3_DOUBLE_CLICKED | BUTTON4_DOUBLE_CLICKED)
+#define BUTTON_TRIPLE_CLICKED (BUTTON1_TRIPLE_CLICKED | BUTTON2_TRIPLE_CLICKED | BUTTON3_TRIPLE_CLICKED | BUTTON4_TRIPLE_CLICKED)
+#define MAX_BUTTONS  4
+#else
+#define BUTTON_CLICKED        (BUTTON1_CLICKED        | BUTTON2_CLICKED        | BUTTON3_CLICKED        | BUTTON4_CLICKED        | BUTTON5_CLICKED)
+#define BUTTON_PRESSED        (BUTTON1_PRESSED        | BUTTON2_PRESSED        | BUTTON3_PRESSED        | BUTTON4_PRESSED        | BUTTON5_PRESSED)
+#define BUTTON_RELEASED       (BUTTON1_RELEASED       | BUTTON2_RELEASED       | BUTTON3_RELEASED       | BUTTON4_RELEASED       | BUTTON5_RELEASED)
+#define BUTTON_DOUBLE_CLICKED (BUTTON1_DOUBLE_CLICKED | BUTTON2_DOUBLE_CLICKED | BUTTON3_DOUBLE_CLICKED | BUTTON4_DOUBLE_CLICKED | BUTTON5_DOUBLE_CLICKED)
+#define BUTTON_TRIPLE_CLICKED (BUTTON1_TRIPLE_CLICKED | BUTTON2_TRIPLE_CLICKED | BUTTON3_TRIPLE_CLICKED | BUTTON4_TRIPLE_CLICKED | BUTTON5_TRIPLE_CLICKED)
+#define MAX_BUTTONS  5
+#endif
 
 #define INVALID_EVENT	-1
 #define NORMAL_EVENT	0
 
 #if USE_GPM_SUPPORT
-#ifndef LINT
-static Gpm_Connect gpm_connect;
-#endif
+
+#ifndef LIBGPM_SONAME
+#define LIBGPM_SONAME "libgpm.so"
 #endif
 
-static mmask_t eventmask;	/* current event mask */
+#define GET_DLSYM(name) (my_##name = (TYPE_##name) dlsym(obj, #name))
+
+#endif				/* USE_GPM_SUPPORT */
 
 static bool _nc_mouse_parse(int);
 static void _nc_mouse_resume(SCREEN *);
@@ -131,18 +160,15 @@ static void _nc_mouse_wrap(SCREEN *);
 
 /* maintain a circular list of mouse events */
 
-/* The definition of the circular list size (EV_MAX), is in curses.priv.h, so
- * wgetch() may refer to the size and call _nc_mouse_parse() before circular
- * list overflow.
- */
-static MEVENT events[EV_MAX];	/* hold the last mouse event seen */
-static MEVENT *eventp = events;	/* next free slot in event queue */
-
 #undef  NEXT
-#define NEXT(ep)	((ep == events + EV_MAX - 1) ? events : ep + 1)
+#define NEXT(ep)	((ep == SP->_mouse_events + EV_MAX - 1) \
+			 ? SP->_mouse_events \
+			 : ep + 1)
 
 #undef  PREV
-#define PREV(ep)	((ep == events) ? events + EV_MAX - 1 : ep - 1)
+#define PREV(ep)	((ep == SP->_mouse_events) \
+			 ? SP->_mouse_events + EV_MAX - 1 \
+			 : ep - 1)
 
 #ifdef TRACE
 static void
@@ -152,9 +178,9 @@ _trace_slot(const char *tag)
 
     _tracef(tag);
 
-    for (ep = events; ep < events + EV_MAX; ep++)
+    for (ep = SP->_mouse_events; ep < SP->_mouse_events + EV_MAX; ep++)
 	_tracef("mouse event queue slot %ld = %s",
-		(long) (ep - events),
+		(long) (ep - SP->_mouse_events),
 		_tracemouse(ep));
 }
 #endif
@@ -164,12 +190,6 @@ _trace_slot(const char *tag)
 #  define TOP_ROW          0
 #  define LEFT_COL         0
 
-static int mouse_wfd;
-static int mouse_thread;
-static int mouse_activated;
-static char mouse_buttons[] =
-{0, 1, 3, 2};
-
 #  define M_FD(sp) sp->_mouse_fd
 
 static void
@@ -182,7 +202,7 @@ write_event(int down, int button, int x, int y)
     buf[3] = ' ' + (button - 1) + (down ? 0 : 0x40);
     buf[4] = ' ' + x - LEFT_COL + 1;
     buf[5] = ' ' + y - TOP_ROW + 1;
-    DosWrite(mouse_wfd, buf, 6, &ignore);
+    DosWrite(SP->_emxmouse_wfd, buf, 6, &ignore);
 }
 
 static void
@@ -214,7 +234,7 @@ mouse_server(unsigned long ignored GCC_UNUSED)
 		    sprintf(err, "Error reading mouse queue, rc=%lu.\r\n", rc);
 		    break;
 		}
-		if (!mouse_activated)
+		if (!SP->_emxmouse_activated)
 		    goto finish;
 
 		/*
@@ -226,13 +246,13 @@ mouse_server(unsigned long ignored GCC_UNUSED)
 		 */
 		if ((mouev.fs ^ oldstate) & MOUSE_BN1_DOWN)
 		    write_event(mouev.fs & MOUSE_BN1_DOWN,
-				mouse_buttons[1], mouev.col, mouev.row);
+				SP->_emxmouse_buttons[1], mouev.col, mouev.row);
 		if ((mouev.fs ^ oldstate) & MOUSE_BN2_DOWN)
 		    write_event(mouev.fs & MOUSE_BN2_DOWN,
-				mouse_buttons[3], mouev.col, mouev.row);
+				SP->_emxmouse_buttons[3], mouev.col, mouev.row);
 		if ((mouev.fs ^ oldstate) & MOUSE_BN3_DOWN)
 		    write_event(mouev.fs & MOUSE_BN3_DOWN,
-				mouse_buttons[2], mouev.col, mouev.row);
+				SP->_emxmouse_buttons[2], mouev.col, mouev.row);
 
 	      finish:
 		oldstate = mouev.fs;
@@ -247,12 +267,6 @@ mouse_server(unsigned long ignored GCC_UNUSED)
     DosExit(EXIT_THREAD, 0L);
 }
 
-static void
-server_state(const int state)
-{				/* It would be nice to implement pointer-off and stop looping... */
-    mouse_activated = state;
-}
-
 #endif /* USE_EMX_MOUSE */
 
 #if USE_SYSMOUSE
@@ -312,9 +326,7 @@ handle_sysmouse(int sig GCC_UNUSED)
 	work->y = the_mouse.u.data.y / SP->_sysmouse_char_height;
     }
 }
-#endif
-
-static int initialized;
+#endif /* USE_SYSMOUSE */
 
 static void
 init_xterm_mouse(void)
@@ -325,77 +337,161 @@ init_xterm_mouse(void)
 	SP->_mouse_xtermcap = "\033[?1000%?%p1%{1}%=%th%el%;";
 }
 
-#if !USE_EMX_MOUSE
 static void
 enable_xterm_mouse(int enable)
 {
-    putp(tparm(SP->_mouse_xtermcap, enable));
+#if USE_EMX_MOUSE
+    SP->_emxmouse_activated = enable;
+#else
+    putp(TPARM_1(SP->_mouse_xtermcap, enable));
+#endif
+    SP->_mouse_active = enable;
 }
-#endif /* !USE_EMX_MOUSE */
+
+#if USE_GPM_SUPPORT
+static int
+allow_gpm_mouse(void)
+{
+    /* GPM does printf's without checking if stdout is a terminal */
+    if (isatty(fileno(stdout))) {
+	char *env = getenv("TERM");
+	/* GPM checks the beginning of the $TERM variable to decide if
+	 * it should pass xterm events through.  There is no real advantage
+	 * in allowing GPM to do this.
+	 */
+	if (env == 0 || strncmp(env, "xterm", 5))
+	    return TRUE;
+    }
+    return FALSE;
+}
+
+static bool
+enable_gpm_mouse(int enable)
+{
+    bool result;
+
+    T((T_CALLED("enable_gpm_mouse(%d)"), enable));
+
+    if (enable && !SP->_mouse_active) {
+	/* GPM: initialize connection to gpm server */
+	SP->_mouse_gpm_connect.eventMask = GPM_DOWN | GPM_UP;
+	SP->_mouse_gpm_connect.defaultMask =
+	    ~(SP->_mouse_gpm_connect.eventMask | GPM_HARD);
+	SP->_mouse_gpm_connect.minMod = 0;
+	SP->_mouse_gpm_connect.maxMod =
+	    (unsigned short) (~((1 << KG_SHIFT) |
+				(1 << KG_SHIFTL) |
+				(1 << KG_SHIFTR)));
+	/*
+	 * Note: GPM hardcodes \E[?1001s and \E[?1000h during its open.
+	 * The former is recognized by wscons (SunOS), and the latter by
+	 * xterm.  Those will not show up in ncurses' traces.
+	 */
+	result = (my_Gpm_Open(&SP->_mouse_gpm_connect, 0) >= 0);
+	SP->_mouse_active = result;
+	T(("GPM open %s", result ? "succeeded" : "failed"));
+    } else {
+	if (!enable && SP->_mouse_active) {
+	    /* GPM: close connection to gpm server */
+	    my_Gpm_Close();
+	    SP->_mouse_active = FALSE;
+	    T(("GPM closed"));
+	}
+	result = FALSE;
+    }
+    returnBool(result);
+}
+#endif /* USE_GPM_SUPPORT */
+
+#define xterm_kmous "\033[M"
 
 static void
 initialize_mousetype(void)
 {
-    static const char *xterm_kmous = "\033[M";
+    T((T_CALLED("initialize_mousetype()")));
 
     /* Try gpm first, because gpm may be configured to run in xterm */
 #if USE_GPM_SUPPORT
-    /* GPM does printf's without checking if stdout is a terminal */
-    if (isatty(fileno(stdout))) {
-	/* GPM: initialize connection to gpm server */
-	gpm_connect.eventMask = GPM_DOWN | GPM_UP;
-	gpm_connect.defaultMask = ~(gpm_connect.eventMask | GPM_HARD);
-	gpm_connect.minMod = 0;
-	gpm_connect.maxMod = ~((1 << KG_SHIFT) | (1 << KG_SHIFTL) | (1 << KG_SHIFTR));
-	if (Gpm_Open(&gpm_connect, 0) >= 0) {	/* returns the file-descriptor */
+    if (allow_gpm_mouse()) {
+	if (!SP->_mouse_gpm_loaded) {
+#ifdef HAVE_LIBDL
+	    void *obj;
+
+	    if ((obj = dlopen(LIBGPM_SONAME, my_RTLD)) != 0) {
+		if (GET_DLSYM(gpm_fd) == 0 ||
+		    GET_DLSYM(Gpm_Open) == 0 ||
+		    GET_DLSYM(Gpm_Close) == 0 ||
+		    GET_DLSYM(Gpm_GetEvent) == 0) {
+		    T(("GPM initialization failed: %s", dlerror()));
+		    dlclose(obj);
+		} else {
+		    SP->_mouse_gpm_found = TRUE;
+		}
+	    }
+#else /* !HAVE_LIBDL */
+	    SP->_mouse_gpm_found = TRUE;
+#endif
+	    SP->_mouse_gpm_loaded = TRUE;
+	}
+
+	/*
+	 * The gpm_fd file-descriptor may be negative (xterm).  So we have to
+	 * maintain our notion of whether the mouse connection is active
+	 * without testing the file-descriptor.
+	 */
+	if (SP->_mouse_gpm_found && enable_gpm_mouse(TRUE)) {
 	    SP->_mouse_type = M_GPM;
-	    SP->_mouse_fd = gpm_fd;
-	    return;
+	    SP->_mouse_fd = *(my_gpm_fd);
+	    T(("GPM mouse_fd %d", SP->_mouse_fd));
+	    returnVoid;
 	}
     }
-#endif
+#endif /* USE_GPM_SUPPORT */
 
     /* OS/2 VIO */
 #if USE_EMX_MOUSE
-    if (!mouse_thread
+    if (!SP->_emxmouse_thread
 	&& strstr(cur_term->type.term_names, "xterm") == 0
 	&& key_mouse) {
 	int handles[2];
 
 	if (pipe(handles) < 0) {
 	    perror("mouse pipe error");
-	    return;
+	    returnVoid;
 	} else {
 	    int rc;
 
-	    if (!mouse_buttons[0]) {
+	    if (!SP->_emxmouse_buttons[0]) {
 		char *s = getenv("MOUSE_BUTTONS_123");
 
-		mouse_buttons[0] = 1;
+		SP->_emxmouse_buttons[0] = 1;
 		if (s && strlen(s) >= 3) {
-		    mouse_buttons[1] = s[0] - '0';
-		    mouse_buttons[2] = s[1] - '0';
-		    mouse_buttons[3] = s[2] - '0';
+		    SP->_emxmouse_buttons[1] = s[0] - '0';
+		    SP->_emxmouse_buttons[2] = s[1] - '0';
+		    SP->_emxmouse_buttons[3] = s[2] - '0';
+		} else {
+		    SP->_emxmouse_buttons[1] = 1;
+		    SP->_emxmouse_buttons[2] = 3;
+		    SP->_emxmouse_buttons[3] = 2;
 		}
 	    }
-	    mouse_wfd = handles[1];
+	    SP->_emxmouse_wfd = handles[1];
 	    M_FD(SP) = handles[0];
 	    /* Needed? */
 	    setmode(handles[0], O_BINARY);
 	    setmode(handles[1], O_BINARY);
 	    /* Do not use CRT functions, we may single-threaded. */
-	    rc = DosCreateThread((unsigned long *) &mouse_thread,
+	    rc = DosCreateThread((unsigned long *) &SP->_emxmouse_thread,
 				 mouse_server, 0, 0, 8192);
 	    if (rc) {
 		printf("mouse thread error %d=%#x", rc, rc);
-		return;
 	    } else {
 		SP->_mouse_type = M_XTERM;
-		return;
 	    }
+	    returnVoid;
 	}
     }
-#endif
+#endif /* USE_EMX_MOUSE */
 
 #if USE_SYSMOUSE
     {
@@ -456,7 +552,7 @@ initialize_mousetype(void)
 		if (SP->_sysmouse_char_height <= 0)
 		    SP->_sysmouse_char_height = 16;
 		SP->_mouse_type = M_SYSMOUSE;
-		return;
+		returnVoid;
 	    }
 	}
     }
@@ -464,35 +560,41 @@ initialize_mousetype(void)
 
     /* we know how to recognize mouse events under "xterm" */
     if (key_mouse != 0) {
-	if (!strcmp(key_mouse, xterm_kmous)) {
+	if (!strcmp(key_mouse, xterm_kmous)
+	    || strstr(cur_term->type.term_names, "xterm") != 0) {
 	    init_xterm_mouse();
-	    return;
 	}
     } else if (strstr(cur_term->type.term_names, "xterm") != 0) {
-	(void) _nc_add_to_try(&(SP->_keytry), xterm_kmous, KEY_MOUSE);
-	init_xterm_mouse();
-	return;
+	if (_nc_add_to_try(&(SP->_keytry), xterm_kmous, KEY_MOUSE) == OK)
+	    init_xterm_mouse();
     }
+    returnVoid;
 }
 
-static void
+static bool
 _nc_mouse_init(void)
 /* initialize the mouse */
 {
+    bool result = FALSE;
     int i;
 
-    if (!initialized) {
-	initialized = TRUE;
+    if (SP != 0) {
+	if (!SP->_mouse_initialized) {
+	    SP->_mouse_initialized = TRUE;
 
-	TR(MY_TRACE, ("_nc_mouse_init() called"));
+	    TR(MY_TRACE, ("_nc_mouse_init() called"));
 
-	for (i = 0; i < EV_MAX; i++)
-	    events[i].id = INVALID_EVENT;
+	    SP->_mouse_eventp = SP->_mouse_events;
+	    for (i = 0; i < EV_MAX; i++)
+		SP->_mouse_events[i].id = INVALID_EVENT;
 
-	initialize_mousetype();
+	    initialize_mousetype();
 
-	T(("_nc_mouse_init() set mousetype to %d", SP->_mouse_type));
+	    T(("_nc_mouse_init() set mousetype to %d", SP->_mouse_type));
+	}
+	result = SP->_mouse_initialized;
     }
+    return result;
 }
 
 /*
@@ -500,10 +602,13 @@ _nc_mouse_init(void)
  * fifo_push() in lib_getch.c
  */
 static bool
-_nc_mouse_event(SCREEN * sp GCC_UNUSED)
+_nc_mouse_event(SCREEN *sp GCC_UNUSED)
 {
+    MEVENT *eventp = SP->_mouse_eventp;
     bool result = FALSE;
 
+    (void) eventp;
+
     switch (SP->_mouse_type) {
     case M_XTERM:
 	/* xterm: never have to query, mouse events are in the keyboard stream */
@@ -530,7 +635,7 @@ _nc_mouse_event(SCREEN * sp GCC_UNUSED)
 	    /* query server for event, return TRUE if we find one */
 	    Gpm_Event ev;
 
-	    if (Gpm_GetEvent(&ev) == 1) {
+	    if (my_Gpm_GetEvent(&ev) == 1) {
 		/* there's only one mouse... */
 		eventp->id = NORMAL_EVENT;
 
@@ -561,7 +666,7 @@ _nc_mouse_event(SCREEN * sp GCC_UNUSED)
 		eventp->z = 0;
 
 		/* bump the next-free pointer into the circular list */
-		eventp = NEXT(eventp);
+		SP->_mouse_eventp = eventp = NEXT(eventp);
 		result = TRUE;
 	    }
 	}
@@ -585,7 +690,7 @@ _nc_mouse_event(SCREEN * sp GCC_UNUSED)
 	    }
 
 	    /* bump the next-free pointer into the circular list */
-	    eventp = NEXT(eventp);
+	    SP->_mouse_eventp = eventp = NEXT(eventp);
 	    result = TRUE;
 	}
 	break;
@@ -599,10 +704,12 @@ _nc_mouse_event(SCREEN * sp GCC_UNUSED)
 }
 
 static bool
-_nc_mouse_inline(SCREEN * sp)
+_nc_mouse_inline(SCREEN *sp)
 /* mouse report received in the keyboard stream -- parse its info */
 {
+    int b;
     bool result = FALSE;
+    MEVENT *eventp = SP->_mouse_eventp;
 
     TR(MY_TRACE, ("_nc_mouse_inline() called"));
 
@@ -639,8 +746,10 @@ _nc_mouse_inline(SCREEN * sp)
 	 * (End quote)  By the time we get here, we've eaten the
 	 * key prefix.  FYI, the loop below is necessary because
 	 * mouse click info isn't guaranteed to present as a
-	 * single clist item.  It always does under Linux but often
-	 * fails to under Solaris.
+	 * single clist item.
+	 *
+	 * Wheel mice may return buttons 4 and 5 when the wheel is turned.
+	 * We encode those as button presses.
 	 */
 	for (grabbed = 0; grabbed < 3; grabbed += res) {
 
@@ -679,11 +788,19 @@ _nc_mouse_inline(SCREEN * sp)
 
 	switch (kbuf[0] & 0x3) {
 	case 0x0:
-	    PRESS_POSITION(1);
+	    if (kbuf[0] & 64)
+		eventp->bstate = MASK_PRESS(4);
+	    else
+		PRESS_POSITION(1);
 	    break;
 
 	case 0x1:
-	    PRESS_POSITION(2);
+#if NCURSES_MOUSE_VERSION == 2
+	    if (kbuf[0] & 64)
+		eventp->bstate = MASK_PRESS(5);
+	    else
+#endif
+		PRESS_POSITION(2);
 	    break;
 
 	case 0x2:
@@ -700,12 +817,10 @@ _nc_mouse_inline(SCREEN * sp)
 	     */
 	    if (prev & (BUTTON_PRESSED | BUTTON_RELEASED)) {
 		eventp->bstate = BUTTON_RELEASED;
-		if (!(prev & BUTTON1_PRESSED))
-		    eventp->bstate &= ~BUTTON1_RELEASED;
-		if (!(prev & BUTTON2_PRESSED))
-		    eventp->bstate &= ~BUTTON2_RELEASED;
-		if (!(prev & BUTTON3_PRESSED))
-		    eventp->bstate &= ~BUTTON3_RELEASED;
+		for (b = 1; b <= MAX_BUTTONS; ++b) {
+		    if (!(prev & MASK_PRESS(b)))
+			eventp->bstate &= ~MASK_RELEASE(b);
+		}
 	    } else {
 		/*
 		 * XFree86 xterm will return a stream of release-events to
@@ -733,10 +848,10 @@ _nc_mouse_inline(SCREEN * sp)
 	TR(MY_TRACE,
 	   ("_nc_mouse_inline: primitive mouse-event %s has slot %ld",
 	    _tracemouse(eventp),
-	    (long) (eventp - events)));
+	    (long) (eventp - SP->_mouse_events)));
 
 	/* bump the next-free pointer into the circular list */
-	eventp = NEXT(eventp);
+	SP->_mouse_eventp = NEXT(eventp);
 #if 0				/* this return would be needed for QNX's mods to lib_getch.c */
 	return (TRUE);
 #endif
@@ -748,10 +863,11 @@ _nc_mouse_inline(SCREEN * sp)
 static void
 mouse_activate(bool on)
 {
-    if (!on && !initialized)
+    if (!on && !SP->_mouse_initialized)
 	return;
 
-    _nc_mouse_init();
+    if (!_nc_mouse_init())
+	return;
 
     if (on) {
 
@@ -761,20 +877,20 @@ mouse_activate(bool on)
 	    keyok(KEY_MOUSE, on);
 #endif
 	    TPUTS_TRACE("xterm mouse initialization");
-#if USE_EMX_MOUSE
-	    server_state(1);
-#else
 	    enable_xterm_mouse(1);
-#endif
 	    break;
 #if USE_GPM_SUPPORT
 	case M_GPM:
-	    SP->_mouse_fd = gpm_fd;
+	    if (enable_gpm_mouse(1)) {
+		SP->_mouse_fd = *(my_gpm_fd);
+		T(("GPM mouse_fd %d", SP->_mouse_fd));
+	    }
 	    break;
 #endif
 #if USE_SYSMOUSE
 	case M_SYSMOUSE:
 	    signal(SIGUSR2, handle_sysmouse);
+	    SP->_mouse_active = TRUE;
 	    break;
 #endif
 	case M_NONE:
@@ -788,25 +904,22 @@ mouse_activate(bool on)
 	SP->_mouse_parse = _nc_mouse_parse;
 	SP->_mouse_resume = _nc_mouse_resume;
 	SP->_mouse_wrap = _nc_mouse_wrap;
-
     } else {
 
 	switch (SP->_mouse_type) {
 	case M_XTERM:
 	    TPUTS_TRACE("xterm mouse deinitialization");
-#if USE_EMX_MOUSE
-	    server_state(0);
-#else
 	    enable_xterm_mouse(0);
-#endif
 	    break;
 #if USE_GPM_SUPPORT
 	case M_GPM:
+	    enable_gpm_mouse(0);
 	    break;
 #endif
 #if USE_SYSMOUSE
 	case M_SYSMOUSE:
 	    signal(SIGUSR2, SIG_IGN);
+	    SP->_mouse_active = FALSE;
 	    break;
 #endif
 	case M_NONE:
@@ -826,8 +939,10 @@ static bool
 _nc_mouse_parse(int runcount)
 /* parse a run of atomic mouse events into a gesture */
 {
+    MEVENT *eventp = SP->_mouse_eventp;
     MEVENT *ep, *runp, *next, *prev = PREV(eventp);
     int n;
+    int b;
     bool merge;
 
     TR(MY_TRACE, ("_nc_mouse_parse(%d) called", runcount));
@@ -857,9 +972,9 @@ _nc_mouse_parse(int runcount)
 	TR(MY_TRACE,
 	   ("_nc_mouse_parse: returning simple mouse event %s at slot %ld",
 	    _tracemouse(prev),
-	    (long) (prev - events)));
+	    (long) (prev - SP->_mouse_events)));
 	return (prev->id >= NORMAL_EVENT)
-	    ? ((prev->bstate & eventmask) ? TRUE : FALSE)
+	    ? ((prev->bstate & SP->_mouse_mask) ? TRUE : FALSE)
 	    : FALSE;
     }
 
@@ -873,8 +988,8 @@ _nc_mouse_parse(int runcount)
     if (_nc_tracing & TRACE_IEVENT) {
 	_trace_slot("before mouse press/release merge:");
 	_tracef("_nc_mouse_parse: run starts at %ld, ends at %ld, count %d",
-		(long) (runp - events),
-		(long) ((eventp - events) + (EV_MAX - 1)) % EV_MAX,
+		(long) (runp - SP->_mouse_events),
+		(long) ((eventp - SP->_mouse_events) + (EV_MAX - 1)) % EV_MAX,
 		runcount);
     }
 #endif /* TRACE */
@@ -883,32 +998,27 @@ _nc_mouse_parse(int runcount)
     do {
 	merge = FALSE;
 	for (ep = runp; (next = NEXT(ep)) != eventp; ep = next) {
+
+#define MASK_CHANGED(x) (!(ep->bstate & MASK_PRESS(x)) \
+		      == !(next->bstate & MASK_RELEASE(x)))
+
 	    if (ep->x == next->x && ep->y == next->y
 		&& (ep->bstate & BUTTON_PRESSED)
-		&& (!(ep->bstate & BUTTON1_PRESSED)
-		    == !(next->bstate & BUTTON1_RELEASED))
-		&& (!(ep->bstate & BUTTON2_PRESSED)
-		    == !(next->bstate & BUTTON2_RELEASED))
-		&& (!(ep->bstate & BUTTON3_PRESSED)
-		    == !(next->bstate & BUTTON3_RELEASED))
+		&& MASK_CHANGED(1)
+		&& MASK_CHANGED(2)
+		&& MASK_CHANGED(3)
+		&& MASK_CHANGED(4)
+#if NCURSES_MOUSE_VERSION == 2
+		&& MASK_CHANGED(5)
+#endif
 		) {
-		if ((eventmask & BUTTON1_CLICKED)
-		    && (ep->bstate & BUTTON1_PRESSED)) {
-		    ep->bstate &= ~BUTTON1_PRESSED;
-		    ep->bstate |= BUTTON1_CLICKED;
-		    merge = TRUE;
-		}
-		if ((eventmask & BUTTON2_CLICKED)
-		    && (ep->bstate & BUTTON2_PRESSED)) {
-		    ep->bstate &= ~BUTTON2_PRESSED;
-		    ep->bstate |= BUTTON2_CLICKED;
-		    merge = TRUE;
-		}
-		if ((eventmask & BUTTON3_CLICKED)
-		    && (ep->bstate & BUTTON3_PRESSED)) {
-		    ep->bstate &= ~BUTTON3_PRESSED;
-		    ep->bstate |= BUTTON3_CLICKED;
-		    merge = TRUE;
+		for (b = 1; b <= MAX_BUTTONS; ++b) {
+		    if ((SP->_mouse_mask & MASK_CLICK(b))
+			&& (ep->bstate & MASK_PRESS(b))) {
+			ep->bstate &= ~MASK_PRESS(b);
+			ep->bstate |= MASK_CLICK(b);
+			merge = TRUE;
+		    }
 		}
 		if (merge)
 		    next->id = INVALID_EVENT;
@@ -921,8 +1031,8 @@ _nc_mouse_parse(int runcount)
     if (_nc_tracing & TRACE_IEVENT) {
 	_trace_slot("before mouse click merge:");
 	_tracef("_nc_mouse_parse: run starts at %ld, ends at %ld, count %d",
-		(long) (runp - events),
-		(long) ((eventp - events) + (EV_MAX - 1)) % EV_MAX,
+		(long) (runp - SP->_mouse_events),
+		(long) ((eventp - SP->_mouse_events) + (EV_MAX - 1)) % EV_MAX,
 		runcount);
     }
 #endif /* TRACE */
@@ -958,51 +1068,28 @@ _nc_mouse_parse(int runcount)
 		/* merge click events forward */
 		if ((ep->bstate & BUTTON_CLICKED)
 		    && (follower->bstate & BUTTON_CLICKED)) {
-		    if ((eventmask & BUTTON1_DOUBLE_CLICKED)
-			&& (follower->bstate & BUTTON1_CLICKED)) {
-			follower->bstate &= ~BUTTON1_CLICKED;
-			follower->bstate |= BUTTON1_DOUBLE_CLICKED;
-			merge = TRUE;
-		    }
-		    if ((eventmask & BUTTON2_DOUBLE_CLICKED)
-			&& (follower->bstate & BUTTON2_CLICKED)) {
-			follower->bstate &= ~BUTTON2_CLICKED;
-			follower->bstate |= BUTTON2_DOUBLE_CLICKED;
-			merge = TRUE;
-		    }
-		    if ((eventmask & BUTTON3_DOUBLE_CLICKED)
-			&& (follower->bstate & BUTTON3_CLICKED)) {
-			follower->bstate &= ~BUTTON3_CLICKED;
-			follower->bstate |= BUTTON3_DOUBLE_CLICKED;
-			merge = TRUE;
+		    for (b = 1; b <= MAX_BUTTONS; ++b) {
+			if ((SP->_mouse_mask & MASK_DOUBLE_CLICK(b))
+			    && (follower->bstate & MASK_CLICK(b))) {
+			    follower->bstate &= ~MASK_CLICK(b);
+			    follower->bstate |= MASK_DOUBLE_CLICK(b);
+			    merge = TRUE;
+			}
 		    }
 		    if (merge)
 			ep->id = INVALID_EVENT;
 		}
 
 		/* merge double-click events forward */
-		if ((ep->bstate &
-		     (BUTTON1_DOUBLE_CLICKED
-		      | BUTTON2_DOUBLE_CLICKED
-		      | BUTTON3_DOUBLE_CLICKED))
+		if ((ep->bstate & BUTTON_DOUBLE_CLICKED)
 		    && (follower->bstate & BUTTON_CLICKED)) {
-		    if ((eventmask & BUTTON1_TRIPLE_CLICKED)
-			&& (follower->bstate & BUTTON1_CLICKED)) {
-			follower->bstate &= ~BUTTON1_CLICKED;
-			follower->bstate |= BUTTON1_TRIPLE_CLICKED;
-			merge = TRUE;
-		    }
-		    if ((eventmask & BUTTON2_TRIPLE_CLICKED)
-			&& (follower->bstate & BUTTON2_CLICKED)) {
-			follower->bstate &= ~BUTTON2_CLICKED;
-			follower->bstate |= BUTTON2_TRIPLE_CLICKED;
-			merge = TRUE;
-		    }
-		    if ((eventmask & BUTTON3_TRIPLE_CLICKED)
-			&& (follower->bstate & BUTTON3_CLICKED)) {
-			follower->bstate &= ~BUTTON3_CLICKED;
-			follower->bstate |= BUTTON3_TRIPLE_CLICKED;
-			merge = TRUE;
+		    for (b = 1; b <= MAX_BUTTONS; ++b) {
+			if ((SP->_mouse_mask & MASK_TRIPLE_CLICK(b))
+			    && (follower->bstate & MASK_CLICK(b))) {
+			    follower->bstate &= ~MASK_CLICK(b);
+			    follower->bstate |= MASK_TRIPLE_CLICK(b);
+			    merge = TRUE;
+			}
 		    }
 		    if (merge)
 			ep->id = INVALID_EVENT;
@@ -1015,8 +1102,8 @@ _nc_mouse_parse(int runcount)
     if (_nc_tracing & TRACE_IEVENT) {
 	_trace_slot("before mouse event queue compaction:");
 	_tracef("_nc_mouse_parse: run starts at %ld, ends at %ld, count %d",
-		(long) (runp - events),
-		(long) ((eventp - events) + (EV_MAX - 1)) % EV_MAX,
+		(long) (runp - SP->_mouse_events),
+		(long) ((eventp - SP->_mouse_events) + (EV_MAX - 1)) % EV_MAX,
 		runcount);
     }
 #endif /* TRACE */
@@ -1026,15 +1113,15 @@ _nc_mouse_parse(int runcount)
      * don't match the current event mask.
      */
     for (; runcount; prev = PREV(eventp), runcount--)
-	if (prev->id == INVALID_EVENT || !(prev->bstate & eventmask)) {
-	    eventp = prev;
+	if (prev->id == INVALID_EVENT || !(prev->bstate & SP->_mouse_mask)) {
+	    SP->_mouse_eventp = eventp = prev;
 	}
 #ifdef TRACE
     if (_nc_tracing & TRACE_IEVENT) {
 	_trace_slot("after mouse event queue compaction:");
 	_tracef("_nc_mouse_parse: run starts at %ld, ends at %ld, count %d",
-		(long) (runp - events),
-		(long) ((eventp - events) + (EV_MAX - 1)) % EV_MAX,
+		(long) (runp - SP->_mouse_events),
+		(long) ((eventp - SP->_mouse_events) + (EV_MAX - 1)) % EV_MAX,
 		runcount);
     }
     for (ep = runp; ep != eventp; ep = NEXT(ep))
@@ -1042,7 +1129,7 @@ _nc_mouse_parse(int runcount)
 	    TR(MY_TRACE,
 	       ("_nc_mouse_parse: returning composite mouse event %s at slot %ld",
 		_tracemouse(ep),
-		(long) (ep - events)));
+		(long) (ep - SP->_mouse_events)));
 #endif /* TRACE */
 
     /* after all this, do we have a valid event? */
@@ -1050,19 +1137,21 @@ _nc_mouse_parse(int runcount)
 }
 
 static void
-_nc_mouse_wrap(SCREEN * sp GCC_UNUSED)
+_nc_mouse_wrap(SCREEN *sp GCC_UNUSED)
 /* release mouse -- called by endwin() before shellout/exit */
 {
     TR(MY_TRACE, ("_nc_mouse_wrap() called"));
 
     switch (SP->_mouse_type) {
     case M_XTERM:
-	if (eventmask)
+	if (SP->_mouse_mask)
 	    mouse_activate(FALSE);
 	break;
 #if USE_GPM_SUPPORT
 	/* GPM: pass all mouse events to next client */
     case M_GPM:
+	if (SP->_mouse_mask)
+	    mouse_activate(FALSE);
 	break;
 #endif
 #if USE_SYSMOUSE
@@ -1076,7 +1165,7 @@ _nc_mouse_wrap(SCREEN * sp GCC_UNUSED)
 }
 
 static void
-_nc_mouse_resume(SCREEN * sp GCC_UNUSED)
+_nc_mouse_resume(SCREEN *sp GCC_UNUSED)
 /* re-connect to mouse -- called by doupdate() after shellout */
 {
     TR(MY_TRACE, ("_nc_mouse_resume() called"));
@@ -1084,13 +1173,15 @@ _nc_mouse_resume(SCREEN * sp GCC_UNUSED)
     switch (SP->_mouse_type) {
     case M_XTERM:
 	/* xterm: re-enable reporting */
-	if (eventmask)
+	if (SP->_mouse_mask)
 	    mouse_activate(TRUE);
 	break;
 
 #if USE_GPM_SUPPORT
     case M_GPM:
 	/* GPM: reclaim our event set */
+	if (SP->_mouse_mask)
+	    mouse_activate(TRUE);
 	break;
 #endif
 
@@ -1116,7 +1207,8 @@ getmouse(MEVENT * aevent)
 {
     T((T_CALLED("getmouse(%p)"), aevent));
 
-    if (aevent && (SP->_mouse_type != M_NONE)) {
+    if ((aevent != 0) && (SP != 0) && (SP->_mouse_type != M_NONE)) {
+	MEVENT *eventp = SP->_mouse_eventp;
 	/* compute the current-event pointer */
 	MEVENT *prev = PREV(eventp);
 
@@ -1125,7 +1217,7 @@ getmouse(MEVENT * aevent)
 
 	TR(TRACE_IEVENT, ("getmouse: returning event %s from slot %ld",
 			  _tracemouse(prev),
-			  (long) (prev - events)));
+			  (long) (prev - SP->_mouse_events)));
 
 	prev->id = INVALID_EVENT;	/* so the queue slot becomes free */
 	returnCode(OK);
@@ -1137,16 +1229,23 @@ NCURSES_EXPORT(int)
 ungetmouse(MEVENT * aevent)
 /* enqueue a synthesized mouse event to be seen by the next wgetch() */
 {
+    int result = ERR;
+
     T((T_CALLED("ungetmouse(%p)"), aevent));
 
-    /* stick the given event in the next-free slot */
-    *eventp = *aevent;
+    if (aevent != 0 && SP != 0) {
+	MEVENT *eventp = SP->_mouse_eventp;
 
-    /* bump the next-free pointer into the circular list */
-    eventp = NEXT(eventp);
+	/* stick the given event in the next-free slot */
+	*eventp = *aevent;
 
-    /* push back the notification event on the keyboard queue */
-    returnCode(ungetch(KEY_MOUSE));
+	/* bump the next-free pointer into the circular list */
+	SP->_mouse_eventp = NEXT(eventp);
+
+	/* push back the notification event on the keyboard queue */
+	result = ungetch(KEY_MOUSE);
+    }
+    returnCode(result);
 }
 
 NCURSES_EXPORT(mmask_t)
@@ -1155,30 +1254,35 @@ mousemask(mmask_t newmask, mmask_t * oldmask)
 {
     mmask_t result = 0;
 
-    T((T_CALLED("mousemask(%#lx,%p)"), newmask, oldmask));
+    T((T_CALLED("mousemask(%#lx,%p)"), (unsigned long) newmask, oldmask));
 
-    if (oldmask)
-	*oldmask = eventmask;
+    if (SP != 0) {
+	if (oldmask)
+	    *oldmask = SP->_mouse_mask;
 
-    if (!newmask && !initialized)
-	returnBits(0);
+	if (newmask || SP->_mouse_initialized) {
+	    _nc_mouse_init();
+	    if (SP->_mouse_type != M_NONE) {
+		result = newmask &
+		    (REPORT_MOUSE_POSITION
+		     | BUTTON_ALT
+		     | BUTTON_CTRL
+		     | BUTTON_SHIFT
+		     | BUTTON_PRESSED
+		     | BUTTON_RELEASED
+		     | BUTTON_CLICKED
+		     | BUTTON_DOUBLE_CLICKED
+		     | BUTTON_TRIPLE_CLICKED);
 
-    _nc_mouse_init();
-    if (SP->_mouse_type != M_NONE) {
-	eventmask = newmask &
-	    (REPORT_MOUSE_POSITION | BUTTON_ALT | BUTTON_CTRL | BUTTON_SHIFT
-	     | BUTTON_PRESSED
-	     | BUTTON_RELEASED
-	     | BUTTON_CLICKED
-	     | BUTTON1_DOUBLE_CLICKED | BUTTON1_TRIPLE_CLICKED
-	     | BUTTON2_DOUBLE_CLICKED | BUTTON2_TRIPLE_CLICKED
-	     | BUTTON3_DOUBLE_CLICKED | BUTTON3_TRIPLE_CLICKED);
+		mouse_activate((bool) (result != 0));
 
-	mouse_activate(eventmask != 0);
-
-	result = eventmask;
+		SP->_mouse_mask = result;
+	    }
+	}
+    } else {
+	if (oldmask)
+	    *oldmask = SP->_mouse_mask;
     }
-
     returnBits(result);
 }
 
