@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2002,2006 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,6 +29,7 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Thomas E. Dickey                        1996-on                 *
  ****************************************************************************/
 
 /*
@@ -47,45 +48,52 @@
 
 #include <term.h>		/* lines, columns, cur_term */
 
-MODULE_ID("$Id: lib_restart.c,v 1.3 2000/09/02 18:09:44 tom Exp $")
+MODULE_ID("$Id: lib_restart.c,v 1.6 2006/01/14 15:58:23 tom Exp $")
 
-int
+NCURSES_EXPORT(int)
 restartterm(NCURSES_CONST char *termp, int filenum, int *errret)
 {
     int saveecho = SP->_echo;
     int savecbreak = SP->_cbreak;
     int saveraw = SP->_raw;
     int savenl = SP->_nl;
+    int result;
 
     T((T_CALLED("restartterm(%s,%d,%p)"), termp, filenum, errret));
 
-    setupterm(termp, filenum, errret);
-
-    if (saveecho)
-	echo();
-    else
-	noecho();
-
-    if (savecbreak) {
-	cbreak();
-	noraw();
-    } else if (saveraw) {
-	nocbreak();
-	raw();
+    _nc_handle_sigwinch(0);
+    if (setupterm(termp, filenum, errret) != OK) {
+	result = ERR;
     } else {
-	nocbreak();
-	noraw();
-    }
-    if (savenl)
-	nl();
-    else
-	nonl();
 
-    reset_prog_mode();
+	if (saveecho)
+	    echo();
+	else
+	    noecho();
+
+	if (savecbreak) {
+	    cbreak();
+	    noraw();
+	} else if (saveraw) {
+	    nocbreak();
+	    raw();
+	} else {
+	    nocbreak();
+	    noraw();
+	}
+	if (savenl)
+	    nl();
+	else
+	    nonl();
+
+	reset_prog_mode();
 
 #if USE_SIZECHANGE
-    _nc_update_screensize();
+	_nc_update_screensize();
 #endif
 
-    returnCode(OK);
+	result = OK;
+    }
+    _nc_handle_sigwinch(1);
+    returnCode(result);
 }
