@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2001,2002 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2006,2007 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,6 +29,7 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Thomas E. Dickey                        1996-on                 *
  ****************************************************************************/
 
 /*
@@ -43,32 +44,37 @@
 #include <term.h>		/* cur_term */
 #include <tic.h>
 
-MODULE_ID("$Id: lib_set_term.c,v 1.71 2002/09/14 22:48:00 tom Exp $")
+MODULE_ID("$Id: lib_set_term.c,v 1.100 2007/09/08 21:23:43 tom Exp $")
 
 NCURSES_EXPORT(SCREEN *)
-set_term(SCREEN * screenp)
+set_term(SCREEN *screenp)
 {
     SCREEN *oldSP;
 
     T((T_CALLED("set_term(%p)"), screenp));
 
+    _nc_lock_global(set_SP);
+
     oldSP = SP;
     _nc_set_screen(screenp);
 
     set_curterm(SP->_term);
+#if !USE_REENTRANT
     curscr = SP->_curscr;
     newscr = SP->_newscr;
     stdscr = SP->_stdscr;
     COLORS = SP->_color_count;
     COLOR_PAIRS = SP->_pair_count;
-    memcpy(acs_map, SP->_acs_map, sizeof(SP->_acs_map[0]) * ACS_LEN);
+#endif
+
+    _nc_unlock_global(set_SP);
 
     T((T_RETURN("%p"), oldSP));
     return (oldSP);
 }
 
 static void
-_nc_free_keytry(struct tries *kt)
+_nc_free_keytry(TRIES * kt)
 {
     if (kt != 0) {
 	_nc_free_keytry(kt->child);
@@ -81,12 +87,14 @@ _nc_free_keytry(struct tries *kt)
  * Free the storage associated with the given SCREEN sp.
  */
 NCURSES_EXPORT(void)
-delscreen(SCREEN * sp)
+delscreen(SCREEN *sp)
 {
     SCREEN **scan = &_nc_screen_chain;
+    int i;
 
     T((T_CALLED("delscreen(%p)"), sp));
 
+    _nc_lock_global(set_SP);
     while (*scan) {
 	if (*scan == sp) {
 	    *scan = sp->_next_screen;
@@ -100,8 +108,13 @@ delscreen(SCREEN * sp)
     (void) _nc_freewin(sp->_stdscr);
 
     if (sp->_slk != 0) {
-	FreeIfNeeded(sp->_slk->ent);
-	FreeIfNeeded(sp->_slk->buffer);
+	if (sp->_slk->ent != 0) {
+	    for (i = 0; i < sp->_slk->labcnt; ++i) {
+		FreeIfNeeded(sp->_slk->ent[i].ent_text);
+		FreeIfNeeded(sp->_slk->ent[i].form_text);
+	    }
+	    free(sp->_slk->ent);
+	}
 	free(sp->_slk);
 	sp->_slk = 0;
     }
@@ -112,6 +125,8 @@ delscreen(SCREEN * sp)
     _nc_free_keytry(sp->_key_ok);
     sp->_key_ok = 0;
 
+    FreeIfNeeded(sp->_current_attr);
+
     FreeIfNeeded(sp->_color_table);
     FreeIfNeeded(sp->_color_pairs);
 
@@ -119,6 +134,9 @@ delscreen(SCREEN * sp)
     FreeIfNeeded(sp->newhash);
     FreeIfNeeded(sp->hashtab);
 
+    FreeIfNeeded(sp->_acs_map);
+    FreeIfNeeded(sp->_screen_acs_map);
+
     del_curterm(sp->_term);
 
     /*
@@ -141,28 +159,28 @@ delscreen(SCREEN * sp)
      * multiple references in different screens).
      */
     if (sp == SP) {
+#if !USE_REENTRANT
 	curscr = 0;
 	newscr = 0;
 	stdscr = 0;
 	COLORS = 0;
 	COLOR_PAIRS = 0;
+#endif
 	_nc_set_screen(0);
     }
+    _nc_unlock_global(set_SP);
+
     returnVoid;
 }
 
-static ripoff_t rippedoff[5];
-static ripoff_t *rsp = rippedoff;
-#define N_RIPS SIZEOF(SP->_rippedoff)
-
 static bool
-no_mouse_event(SCREEN * sp GCC_UNUSED)
+no_mouse_event(SCREEN *sp GCC_UNUSED)
 {
     return FALSE;
 }
 
 static bool
-no_mouse_inline(SCREEN * sp GCC_UNUSED)
+no_mouse_inline(SCREEN *sp GCC_UNUSED)
 {
     return FALSE;
 }
@@ -174,12 +192,12 @@ no_mouse_parse(int code GCC_UNUSED)
 }
 
 static void
-no_mouse_resume(SCREEN * sp GCC_UNUSED)
+no_mouse_resume(SCREEN *sp GCC_UNUSED)
 {
 }
 
 static void
-no_mouse_wrap(SCREEN * sp GCC_UNUSED)
+no_mouse_wrap(SCREEN *sp GCC_UNUSED)
 {
 }
 
@@ -203,21 +221,66 @@ extract_fgbg(char *src, int *result)
 }
 #endif
 
-NCURSES_EXPORT(int)
-_nc_setupscreen(short slines, short const scolumns, FILE * output)
+#define ripoff_sp	_nc_prescreen.rsp
+#define ripoff_stack	_nc_prescreen.rippedoff
+
 /* OS-independent screen initializations */
+NCURSES_EXPORT(int)
+_nc_setupscreen(int slines GCC_UNUSED,
+		int scolumns GCC_UNUSED,
+		FILE *output,
+		bool filtered,
+		int slk_format)
 {
     int bottom_stolen = 0;
-    size_t i;
+    bool support_cookies = USE_XMC_SUPPORT;
+    ripoff_t *rop;
+
+    T((T_CALLED("_nc_setupscreen(%d, %d, %p, %d, %d)"),
+       slines, scolumns, output, filtered, slk_format));
 
-    T((T_CALLED("_nc_setupscreen(%d, %d, %p)"), slines, scolumns, output));
     assert(SP == 0);		/* has been reset in newterm() ! */
-    if (!_nc_alloc_screen())
+    if (!_nc_alloc_screen()
+	|| ((SP->_acs_map = typeCalloc(chtype, ACS_LEN)) == 0)
+	|| ((SP->_screen_acs_map = typeCalloc(bool, ACS_LEN)) == 0)) {
 	returnCode(ERR);
+    }
 
+    T(("created SP %p", SP));
     SP->_next_screen = _nc_screen_chain;
     _nc_screen_chain = SP;
 
+    if ((SP->_current_attr = typeCalloc(NCURSES_CH_T, 1)) == 0)
+	returnCode(ERR);
+
+    /*
+     * We should always check the screensize, just in case.
+     */
+    _nc_get_screensize(&slines, &scolumns);
+    SET_LINES(slines);
+    SET_COLS(scolumns);
+    T((T_CREATE("screen %s %dx%d"), termname(), LINES, COLS));
+
+    SP->_filtered = filtered;
+
+    /* implement filter mode */
+    if (filtered) {
+	slines = 1;
+	SET_LINES(slines);
+	clear_screen = 0;
+	cursor_down = parm_down_cursor = 0;
+	cursor_address = 0;
+	cursor_up = parm_up_cursor = 0;
+	row_address = 0;
+
+	cursor_home = carriage_return;
+	T(("filter screensize %dx%d", LINES, COLS));
+    }
+#ifdef __DJGPP__
+    T(("setting output mode to binary"));
+    fflush(output);
+    setmode(output, O_BINARY);
+#endif
     _nc_set_buffer(output, TRUE);
     SP->_term = cur_term;
     SP->_lines = slines;
@@ -341,44 +404,113 @@ _nc_setupscreen(short slines, short const scolumns, FILE * output)
     SP->_panelHook.stdscr_pseudo_panel = (struct panel *) 0;
 
     /*
-     * If we've no magic cookie support, we suppress attributes that xmc
-     * would affect, i.e., the attributes that affect the rendition of a
-     * space.  Note that this impacts the alternate character set mapping
-     * as well.
+     * If we've no magic cookie support, we suppress attributes that xmc would
+     * affect, i.e., the attributes that affect the rendition of a space.
      */
-    if (magic_cookie_glitch > 0) {
+    SP->_ok_attributes = termattrs();
+    if (has_colors()) {
+	SP->_ok_attributes |= A_COLOR;
+    }
+#if USE_XMC_SUPPORT
+    /*
+     * If we have no magic-cookie support compiled-in, or if it is suppressed
+     * in the environment, reset the support-flag.
+     */
+    if (magic_cookie_glitch >= 0) {
+	if (getenv("NCURSES_NO_MAGIC_COOKIE") != 0) {
+	    support_cookies = FALSE;
+	}
+    }
+#endif
 
-	SP->_xmc_triggers = termattrs() & (
-					      A_ALTCHARSET |
-					      A_BLINK |
-					      A_BOLD |
-					      A_REVERSE |
-					      A_STANDOUT |
-					      A_UNDERLINE
+    if (!support_cookies && magic_cookie_glitch >= 0) {
+	T(("will disable attributes to work w/o magic cookies"));
+    }
+
+    if (magic_cookie_glitch > 0) {	/* tvi, wyse */
+
+	SP->_xmc_triggers = SP->_ok_attributes & (
+						     A_STANDOUT |
+						     A_UNDERLINE |
+						     A_REVERSE |
+						     A_BLINK |
+						     A_DIM |
+						     A_BOLD |
+						     A_INVIS |
+						     A_PROTECT
 	    );
+#if 0
+	/*
+	 * We "should" treat colors as an attribute.  The wyse350 (and its
+	 * clones) appear to be the only ones that have both colors and magic
+	 * cookies.
+	 */
+	if (has_colors()) {
+	    SP->_xmc_triggers |= A_COLOR;
+	}
+#endif
 	SP->_xmc_suppress = SP->_xmc_triggers & (chtype) ~(A_BOLD);
 
 	T(("magic cookie attributes %s", _traceattr(SP->_xmc_suppress)));
+	/*
+	 * Supporting line-drawing may be possible.  But make the regular
+	 * video attributes work first.
+	 */
+	acs_chars = ABSENT_STRING;
+	ena_acs = ABSENT_STRING;
+	enter_alt_charset_mode = ABSENT_STRING;
+	exit_alt_charset_mode = ABSENT_STRING;
 #if USE_XMC_SUPPORT
 	/*
-	 * To keep this simple, suppress all of the optimization hooks
-	 * except for clear_screen and the cursor addressing.
+	 * To keep the cookie support simple, suppress all of the optimization
+	 * hooks except for clear_screen and the cursor addressing.
 	 */
-	clr_eol = 0;
-	clr_eos = 0;
-	set_attributes = 0;
-#else
-	magic_cookie_glitch = ABSENT_NUMERIC;
-	acs_chars = 0;
+	if (support_cookies) {
+	    clr_eol = ABSENT_STRING;
+	    clr_eos = ABSENT_STRING;
+	    set_attributes = ABSENT_STRING;
+	}
 #endif
+    } else if (magic_cookie_glitch == 0) {	/* hpterm */
+    }
+
+    /*
+     * If magic cookies are not supported, cancel the strings that set
+     * video attributes.
+     */
+    if (!support_cookies && magic_cookie_glitch >= 0) {
+	magic_cookie_glitch = ABSENT_NUMERIC;
+	set_attributes = ABSENT_STRING;
+	enter_blink_mode = ABSENT_STRING;
+	enter_bold_mode = ABSENT_STRING;
+	enter_dim_mode = ABSENT_STRING;
+	enter_reverse_mode = ABSENT_STRING;
+	enter_standout_mode = ABSENT_STRING;
+	enter_underline_mode = ABSENT_STRING;
     }
 
     /* initialize normal acs before wide, since we use mapping in the latter */
+#if !USE_WIDEC_SUPPORT
+    if (_nc_unicode_locale() && _nc_locale_breaks_acs()) {
+	acs_chars = NULL;
+	ena_acs = NULL;
+	enter_alt_charset_mode = NULL;
+	exit_alt_charset_mode = NULL;
+	set_attributes = NULL;
+    }
+#endif
     _nc_init_acs();
 #if USE_WIDEC_SUPPORT
     _nc_init_wacs();
+
+    SP->_screen_acs_fix = (_nc_unicode_locale() && _nc_locale_breaks_acs());
+    {
+	char *env = _nc_get_locale();
+	SP->_legacy_coding = ((env == 0)
+			      || !strcmp(env, "C")
+			      || !strcmp(env, "POSIX"));
+    }
 #endif
-    memcpy(SP->_acs_map, acs_map, sizeof(chtype) * ACS_LEN);
 
     _nc_idcok = TRUE;
     _nc_idlok = FALSE;
@@ -389,15 +521,17 @@ _nc_setupscreen(short slines, short const scolumns, FILE * output)
     SP->newhash = 0;
 
     T(("creating newscr"));
-    if ((newscr = newwin(slines, scolumns, 0, 0)) == 0)
+    if ((SP->_newscr = newwin(slines, scolumns, 0, 0)) == 0)
 	returnCode(ERR);
 
     T(("creating curscr"));
-    if ((curscr = newwin(slines, scolumns, 0, 0)) == 0)
+    if ((SP->_curscr = newwin(slines, scolumns, 0, 0)) == 0)
 	returnCode(ERR);
 
-    SP->_newscr = newscr;
-    SP->_curscr = curscr;
+#if !USE_REENTRANT
+    newscr = SP->_newscr;
+    curscr = SP->_curscr;
+#endif
 #if USE_SIZECHANGE
     SP->_resize = resizeterm;
 #endif
@@ -408,46 +542,63 @@ _nc_setupscreen(short slines, short const scolumns, FILE * output)
     def_shell_mode();
     def_prog_mode();
 
-    for (i = 0, rsp = rippedoff; rsp->line && (i < N_RIPS); rsp++, i++) {
-	T(("ripping off line %d at %s", i, rsp->line < 0 ? "bottom" : "top"));
-	SP->_rippedoff[i] = rippedoff[i];
-	if (rsp->hook) {
-	    int count = (rsp->line < 0) ? -rsp->line : rsp->line;
+    for (rop = ripoff_stack;
+	 rop != ripoff_sp && (rop - ripoff_stack) < N_RIPS;
+	 rop++) {
 
-	    SP->_rippedoff[i].w = newwin(count,
-					 scolumns,
-					 ((rsp->line < 0)
-					  ? SP->_lines_avail - count
-					  : 0),
-					 0);
-	    if (SP->_rippedoff[i].w != 0)
-		SP->_rippedoff[i].hook(SP->_rippedoff[i].w, scolumns);
+	/* If we must simulate soft labels, grab off the line to be used.
+	   We assume that we must simulate, if it is none of the standard
+	   formats (4-4 or 3-2-3) for which there may be some hardware
+	   support. */
+	if (rop->hook == _nc_slk_initialize)
+	    if (!(num_labels <= 0 || !SLK_STDFMT(slk_format)))
+		continue;
+	if (rop->hook) {
+	    int count;
+	    WINDOW *w;
+
+	    count = (rop->line < 0) ? -rop->line : rop->line;
+	    T(("ripping off %i lines at %s", count,
+	       ((rop->line < 0)
+		? "bottom"
+		: "top")));
+
+	    w = newwin(count, scolumns,
+		       ((rop->line < 0)
+			? SP->_lines_avail - count
+			: 0),
+		       0);
+	    if (w)
+		rop->hook(w, scolumns);
 	    else
 		returnCode(ERR);
-	    if (rsp->line < 0)
+	    if (rop->line < 0)
 		bottom_stolen += count;
 	    else
 		SP->_topstolen += count;
 	    SP->_lines_avail -= count;
 	}
-	rsp->line = 0;
     }
-    SP->_rip_count = i;
     /* reset the stack */
-    rsp = rippedoff;
+    ripoff_sp = ripoff_stack;
 
     T(("creating stdscr"));
     assert((SP->_lines_avail + SP->_topstolen + bottom_stolen) == slines);
-    if ((stdscr = newwin(LINES = SP->_lines_avail, scolumns, 0, 0)) == 0)
+    if ((SP->_stdscr = newwin(SP->_lines_avail, scolumns, 0, 0)) == 0)
 	returnCode(ERR);
-    SP->_stdscr = stdscr;
+
+    SET_LINES(SP->_lines_avail);
+#if !USE_REENTRANT
+    stdscr = SP->_stdscr;
+#endif
 
     returnCode(OK);
 }
 
-/* The internal implementation interprets line as the number of
-   lines to rip off from the top or bottom.
-   */
+/*
+ * The internal implementation interprets line as the number of lines to rip
+ * off from the top or bottom.
+ */
 NCURSES_EXPORT(int)
 _nc_ripoffline(int line, int (*init) (WINDOW *, int))
 {
@@ -455,13 +606,14 @@ _nc_ripoffline(int line, int (*init) (WINDOW *, int))
 
     if (line != 0) {
 
-	if (rsp >= rippedoff + N_RIPS)
+	if (ripoff_sp == 0)
+	    ripoff_sp = ripoff_stack;
+	if (ripoff_sp >= ripoff_stack + N_RIPS)
 	    returnCode(ERR);
 
-	rsp->line = line;
-	rsp->hook = init;
-	rsp->w = 0;
-	rsp++;
+	ripoff_sp->line = line;
+	ripoff_sp->hook = init;
+	ripoff_sp++;
     }
 
     returnCode(OK);
