@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2003,2005 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -28,28 +28,31 @@
 
 /****************************************************************************
  *  Author:  Juergen Pfeifer, 1998                                          *
- *  Contact: http://www.familiepfeifer.de/Contact.aspx?Lang=en              *
+ *     and:  Thomas E. Dickey 2005                                          *
  ****************************************************************************/
 
 /*
  *	lib_slkatr_set.c
  *	Soft key routines.
- *      Set the labels attributes
+ *	Set the label's attributes
  */
 #include <curses.priv.h>
 
-MODULE_ID("$Id: lib_slkatr_set.c,v 1.6 2002/07/06 22:47:23 juergen Exp $")
+MODULE_ID("$Id: lib_slkatr_set.c,v 1.10 2005/01/28 21:11:53 tom Exp $")
 
 NCURSES_EXPORT(int)
-slk_attr_set
-(const attr_t attr, short color_pair_number, void *opts)
+slk_attr_set(const attr_t attr, short color_pair_number, void *opts)
 {
     T((T_CALLED("slk_attr_set(%s,%d)"), _traceattr(attr), color_pair_number));
 
     if (SP != 0 && SP->_slk != 0 && !opts &&
 	color_pair_number >= 0 && color_pair_number < COLOR_PAIRS) {
-	SP->_slk->attr = attr;
-	toggle_attr_on(SP->_slk->attr, COLOR_PAIR(color_pair_number));
+	TR(TRACE_ATTRS, ("... current %s", _tracech_t(CHREF(SP->_slk->attr))));
+	SetAttr(SP->_slk->attr, attr);
+	if (color_pair_number > 0) {
+	    SetPair(SP->_slk->attr, color_pair_number);
+	}
+	TR(TRACE_ATTRS, ("new attribute is %s", _tracech_t(CHREF(SP->_slk->attr))));
 	returnCode(OK);
     } else
 	returnCode(ERR);
