@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2006,2007 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,6 +29,7 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Thomas E. Dickey                        1996-on                 *
  ****************************************************************************/
 
 /*
@@ -69,13 +70,14 @@
 #undef B115200
 #undef B230400
 #undef B460800
+#undef B921600
 #define USE_OLD_TTY
 #include <sys/ttydev.h>
 #else
 #undef USE_OLD_TTY
 #endif /* USE_OLD_TTY */
 
-MODULE_ID("$Id: lib_baudrate.c,v 1.21 2001/06/30 22:59:22 tom Exp $")
+MODULE_ID("$Id: lib_baudrate.c,v 1.24 2007/03/10 19:25:57 tom Exp $")
 
 /*
  *	int
@@ -132,21 +134,28 @@ static struct speed const speeds[] =
 #ifdef B460800
     {B460800, 460800},
 #endif
+#ifdef B921600
+    {B921600, 921600},
+#endif
 };
 
 NCURSES_EXPORT(int)
 _nc_baudrate(int OSpeed)
 {
+#if !USE_REENTRANT
     static int last_OSpeed;
     static int last_baudrate;
+#endif
 
-    int result;
+    int result = ERR;
     unsigned i;
 
+#if !USE_REENTRANT
     if (OSpeed == last_OSpeed) {
 	result = last_baudrate;
-    } else {
-	result = ERR;
+    }
+#endif
+    if (result == ERR) {
 	if (OSpeed >= 0) {
 	    for (i = 0; i < SIZEOF(speeds); i++) {
 		if (speeds[i].s == OSpeed) {
@@ -155,7 +164,12 @@ _nc_baudrate(int OSpeed)
 		}
 	    }
 	}
-	last_baudrate = result;
+#if !USE_REENTRANT
+	if (OSpeed == last_OSpeed) {
+	    last_OSpeed = OSpeed;
+	    last_baudrate = result;
+	}
+#endif
     }
     return (result);
 }
@@ -203,14 +217,14 @@ baudrate(void)
 #ifdef USE_OLD_TTY
     result = cfgetospeed(&cur_term->Nttyb);
     ospeed = _nc_ospeed(result);
-#else
+#else /* !USE_OLD_TTY */
 #ifdef TERMIOS
     ospeed = cfgetospeed(&cur_term->Nttyb);
 #else
     ospeed = cur_term->Nttyb.sg_ospeed;
 #endif
     result = _nc_baudrate(ospeed);
-#endif /* __FreeBSD__ */
+#endif
     if (cur_term != 0)
 	cur_term->_baudrate = result;
 
