@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2005,2006 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -38,49 +38,56 @@
 #include <curses.priv.h>
 #include <term.h>		/* cur_term */
 
-MODULE_ID("$Id: lib_ttyflags.c,v 1.11 2003/05/17 23:50:37 tom Exp $")
-
-#undef tabs
-
-#ifdef TAB3
-# define tabs TAB3
-#else
-# ifdef XTABS
-#  define tabs XTABS
-# else
-#  ifdef OXTABS
-#   define tabs OXTABS
-#  else
-#   define tabs 0
-#  endif
-# endif
-#endif
+MODULE_ID("$Id: lib_ttyflags.c,v 1.13 2006/12/10 01:31:54 tom Exp $")
 
 NCURSES_EXPORT(int)
 _nc_get_tty_mode(TTY * buf)
 {
-    if (cur_term == 0
-	|| GET_TTY(cur_term->Filedes, buf) != 0) {
-	memset(buf, 0, sizeof(*buf));
-	return (ERR);
+    int result = OK;
+
+    if (cur_term == 0) {
+	result = ERR;
+    } else {
+	for (;;) {
+	    if (GET_TTY(cur_term->Filedes, buf) != 0) {
+		if (errno == EINTR)
+		    continue;
+		result = ERR;
+	    }
+	    break;
+	}
     }
+
+    if (result == ERR)
+	memset(buf, 0, sizeof(*buf));
+
     TR(TRACE_BITS, ("_nc_get_tty_mode(%d): %s",
 		    cur_term->Filedes, _nc_trace_ttymode(buf)));
-    return (OK);
+    return (result);
 }
 
 NCURSES_EXPORT(int)
 _nc_set_tty_mode(TTY * buf)
 {
-    if (cur_term == 0
-	|| SET_TTY(cur_term->Filedes, buf) != 0) {
-	if ((errno == ENOTTY) && (SP != 0))
-	    SP->_notty = TRUE;
-	return (ERR);
+    int result = OK;
+
+    if (cur_term == 0) {
+	result = ERR;
+    } else {
+	for (;;) {
+	    if (SET_TTY(cur_term->Filedes, buf) != 0) {
+		if (errno == EINTR)
+		    continue;
+		if ((errno == ENOTTY) && (SP != 0))
+		    SP->_notty = TRUE;
+		result = ERR;
+	    }
+	    break;
+	}
     }
     TR(TRACE_BITS, ("_nc_set_tty_mode(%d): %s",
 		    cur_term->Filedes, _nc_trace_ttymode(buf)));
-    return (OK);
+    return (result);
 }
 
 NCURSES_EXPORT(int)
@@ -89,14 +96,13 @@ def_shell_mode(void)
     T((T_CALLED("def_shell_mode()")));
 
     /*
-     * Turn off the XTABS bit in the tty structure if it was on.  If XTABS
-     * was on, remove the tab and backtab capabilities.
+     * If XTABS was on, remove the tab and backtab capabilities.
      */
 
     if (_nc_get_tty_mode(&cur_term->Ottyb) != OK)
 	returnCode(ERR);
 #ifdef TERMIOS
-    if (cur_term->Ottyb.c_oflag & tabs)
+    if (cur_term->Ottyb.c_oflag & OFLAGS_TABS)
 	tab = back_tab = NULL;
 #else
     if (cur_term->Ottyb.sg_flags & XTABS)
@@ -110,10 +116,14 @@ def_prog_mode(void)
 {
     T((T_CALLED("def_prog_mode()")));
 
+    /*
+     * Turn off the XTABS bit in the tty structure if it was on.
+     */
+
     if (_nc_get_tty_mode(&cur_term->Nttyb) != OK)
 	returnCode(ERR);
 #ifdef TERMIOS
-    cur_term->Nttyb.c_oflag &= ~tabs;
+    cur_term->Nttyb.c_oflag &= ~OFLAGS_TABS;
 #else
     cur_term->Nttyb.sg_flags &= ~XTABS;
 #endif
