@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2006,2007 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,6 +29,7 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Thomas E. Dickey                        1996-on                 *
  ****************************************************************************/
 
 /*
@@ -36,6 +37,7 @@
  */
 
 #include <curses.priv.h>
+#include <hashed_db.h>
 
 #include <sys/stat.h>
 
@@ -46,21 +48,27 @@
 #define S_ISDIR(mode) ((mode & S_IFMT) == S_IFDIR)
 #endif
 
-#if 0
+#if 1
 #define TRACE_OUT(p) DEBUG(2, p)
 #else
 #define TRACE_OUT(p)		/*nothing */
 #endif
 
-MODULE_ID("$Id: write_entry.c,v 1.56 2000/12/10 02:55:08 tom Exp $")
+MODULE_ID("$Id: write_entry.c,v 1.69 2007/05/12 19:04:13 tom Exp $")
 
 static int total_written;
 
-static int write_object(FILE *, TERMTYPE *);
+static int make_db_root(const char *);
+static int write_object(TERMTYPE *, char *, unsigned *, unsigned);
 
+#if !USE_HASHED_DB
 static void
-write_file(char *filename, TERMTYPE * tp)
+write_file(char *filename, TERMTYPE *tp)
 {
+    char buffer[MAX_ENTRY_SIZE];
+    unsigned limit = sizeof(buffer);
+    unsigned offset = 0;
+
     FILE *fp = (_nc_access(filename, W_OK) == 0) ? fopen(filename, "wb") : 0;
     if (fp == 0) {
 	perror(filename);
@@ -68,95 +76,21 @@ write_file(char *filename, TERMTYPE * tp)
     }
     DEBUG(1, ("Created %s", filename));
 
-    if (write_object(fp, tp) == ERR) {
+    if (write_object(tp, buffer, &offset, limit) == ERR
+	|| fwrite(buffer, sizeof(char), offset, fp) != offset) {
 	_nc_syserr_abort("error writing %s/%s", _nc_tic_dir(0), filename);
     }
+
     fclose(fp);
 }
 
 /*
- *	make_directory(char *path)
+ * Check for access rights to destination directories
+ * Create any directories which don't exist.
  *
- *	Make a directory if it doesn't exist.
+ * Note:  there's no reason to return the result of make_db_root(), since
+ * this function is called only in instances where that has to succeed.
  */
-static int
-make_directory(const char *path)
-{
-    int rc;
-    struct stat statbuf;
-    char fullpath[PATH_MAX];
-    const char *destination = _nc_tic_dir(0);
-
-    if (path == destination || *path == '/') {
-	if (strlen(path) + 1 > sizeof(fullpath))
-	    return (-1);
-	(void) strcpy(fullpath, path);
-    } else {
-	if (strlen(destination) + strlen(path) + 2 > sizeof(fullpath))
-	    return (-1);
-	(void) sprintf(fullpath, "%s/%s", destination, path);
-    }
-
-    if ((rc = stat(path, &statbuf)) < 0) {
-	rc = mkdir(path, 0777);
-    } else {
-	if (_nc_access(path, R_OK | W_OK | X_OK) < 0) {
-	    rc = -1;		/* permission denied */
-	} else if (!(S_ISDIR(statbuf.st_mode))) {
-	    rc = -1;		/* not a directory */
-	}
-    }
-    return rc;
-}
-
-NCURSES_EXPORT(void)
-_nc_set_writedir(char *dir)
-/* set the write directory for compiled entries */
-{
-    const char *destination;
-    char actual[PATH_MAX];
-
-    if (dir == 0
-	&& use_terminfo_vars())
-	dir = getenv("TERMINFO");
-
-    if (dir != 0)
-	(void) _nc_tic_dir(dir);
-
-    destination = _nc_tic_dir(0);
-    if (make_directory(destination) < 0) {
-	char *home = _nc_home_terminfo();
-
-	if (home != 0) {
-	    destination = home;
-	    if (make_directory(destination) < 0)
-		_nc_err_abort("%s: permission denied (errno %d)",
-			      destination, errno);
-	}
-    }
-
-    /*
-     * Note: because of this code, this logic should be exercised
-     * *once only* per run.
-     */
-    if (chdir(_nc_tic_dir(destination)) < 0
-	|| getcwd(actual, sizeof(actual)) == 0)
-	_nc_err_abort("%s: not a directory", destination);
-    _nc_keep_tic_dir(strdup(actual));
-}
-
-/*
- *	check_writeable(char code)
- *
- *	Miscellaneous initialisations
- *
- *	Check for access rights to destination directories
- *	Create any directories which don't exist.
- *	Note: there's no reason to return the result of make_directory(), since
- *	this function is called only in instances where that has to succeed.
- *
- */
-
 static void
 check_writeable(int code)
 {
@@ -174,16 +108,124 @@ check_writeable(int code)
 
     dir[0] = code;
     dir[1] = '\0';
-    if (make_directory(dir) < 0) {
+    if (make_db_root(dir) < 0) {
 	_nc_err_abort("%s/%s: permission denied", _nc_tic_dir(0), dir);
     }
 
     verified[s - dirnames] = TRUE;
 }
+#endif /* !USE_HASHED_DB */
+
+static int
+make_db_path(char *dst, const char *src, unsigned limit)
+{
+    int rc = -1;
+    const char *top = _nc_tic_dir(0);
+
+    if (src == top || _nc_is_abs_path(src)) {
+	if (strlen(src) + 1 <= limit) {
+	    (void) strcpy(dst, src);
+	    rc = 0;
+	}
+    } else {
+	if (strlen(top) + strlen(src) + 2 <= limit) {
+	    (void) sprintf(dst, "%s/%s", top, src);
+	    rc = 0;
+	}
+    }
+#if USE_HASHED_DB
+    if (rc == 0) {
+	if (_nc_is_dir_path(dst)) {
+	    rc = -1;
+	} else {
+	    unsigned have = strlen(dst);
+	    if (have > 3 && strcmp(dst + have - 3, DBM_SUFFIX)) {
+		if (have + 3 <= limit)
+		    strcat(dst, DBM_SUFFIX);
+		else
+		    rc = -1;
+	    }
+	}
+    }
+#endif
+    return rc;
+}
+
+/*
+ * Make a database-root if it doesn't exist.
+ */
+static int
+make_db_root(const char *path)
+{
+    int rc;
+    char fullpath[PATH_MAX];
+
+    if ((rc = make_db_path(fullpath, path, sizeof(fullpath))) == 0) {
+#if USE_HASHED_DB
+	DB *capdbp;
+
+	if ((capdbp = _nc_db_open(fullpath, TRUE)) == NULL)
+	    rc = -1;
+	else if (_nc_db_close(capdbp) < 0)
+	    rc = -1;
+#else
+	struct stat statbuf;
+
+	if ((rc = stat(path, &statbuf)) < 0) {
+	    rc = mkdir(path, 0777);
+	} else if (_nc_access(path, R_OK | W_OK | X_OK) < 0) {
+	    rc = -1;		/* permission denied */
+	} else if (!(S_ISDIR(statbuf.st_mode))) {
+	    rc = -1;		/* not a directory */
+	}
+#endif
+    }
+    return rc;
+}
+
+/*
+ * Set the write directory for compiled entries.
+ */
+NCURSES_EXPORT(void)
+_nc_set_writedir(char *dir)
+{
+    const char *destination;
+    char actual[PATH_MAX];
+
+    if (dir == 0
+	&& use_terminfo_vars())
+	dir = getenv("TERMINFO");
+
+    if (dir != 0)
+	(void) _nc_tic_dir(dir);
+
+    destination = _nc_tic_dir(0);
+    if (make_db_root(destination) < 0) {
+	char *home = _nc_home_terminfo();
+
+	if (home != 0) {
+	    destination = home;
+	    if (make_db_root(destination) < 0)
+		_nc_err_abort("%s: permission denied (errno %d)",
+			      destination, errno);
+	}
+    }
+
+    /*
+     * Note: because of this code, this logic should be exercised
+     * *once only* per run.
+     */
+#if USE_HASHED_DB
+    make_db_path(actual, destination, sizeof(actual));
+#else
+    if (chdir(_nc_tic_dir(destination)) < 0
+	|| getcwd(actual, sizeof(actual)) == 0)
+	_nc_err_abort("%s: not a directory", destination);
+#endif
+    _nc_keep_tic_dir(strdup(actual));
+}
 
 /*
- *	_nc_write_entry()
- *
  *	Save the compiled version of a description in the filesystem.
  *
  *	make a copy of the name-list
@@ -203,24 +245,36 @@ check_writeable(int code)
  *	_nc_curr_line is properly set before the write_entry() call.
  */
 
-void
-_nc_write_entry(TERMTYPE * const tp)
+NCURSES_EXPORT(void)
+_nc_write_entry(TERMTYPE *const tp)
 {
+#if USE_HASHED_DB
+
+    char buffer[MAX_ENTRY_SIZE + 1];
+    unsigned limit = sizeof(buffer);
+    unsigned offset = 0;
+
+#else /* !USE_HASHED_DB */
+
     struct stat statbuf;
-    char name_list[MAX_TERMINFO_LENGTH];
-    char *first_name, *other_names;
-    char *ptr;
     char filename[PATH_MAX];
     char linkname[PATH_MAX];
 #if USE_SYMLINKS
     char symlinkname[PATH_MAX];
+#if !HAVE_LINK
+#undef HAVE_LINK
+#define HAVE_LINK 1
+#endif
 #endif /* USE_SYMLINKS */
+
     static int call_count;
     static time_t start_time;	/* time at start of writes */
 
-    if (call_count++ == 0) {
-	start_time = 0;
-    }
+#endif /* USE_HASHED_DB */
+
+    char name_list[MAX_TERMINFO_LENGTH];
+    char *first_name, *other_names;
+    char *ptr;
 
     (void) strcpy(name_list, tp->term_names);
     DEBUG(7, ("Name list = '%s'", name_list));
@@ -252,6 +306,55 @@ _nc_write_entry(TERMTYPE * const tp)
 
     _nc_set_type(first_name);
 
+#if USE_HASHED_DB
+    if (write_object(tp, buffer + 1, &offset, limit - 1) != ERR) {
+	DB *capdb = _nc_db_open(_nc_tic_dir(0), TRUE);
+	DBT key, data;
+
+	if (capdb != 0) {
+	    buffer[0] = 0;
+
+	    memset(&key, 0, sizeof(key));
+	    key.data = tp->term_names;
+	    key.size = strlen(tp->term_names);
+
+	    memset(&data, 0, sizeof(data));
+	    data.data = buffer;
+	    data.size = offset + 1;
+
+	    _nc_db_put(capdb, &key, &data);
+
+	    buffer[0] = 2;
+
+	    key.data = name_list;
+	    key.size = strlen(name_list);
+
+	    strcpy(buffer + 1, tp->term_names);
+	    data.size = strlen(tp->term_names) + 1;
+
+	    _nc_db_put(capdb, &key, &data);
+
+	    while (*other_names != '\0') {
+		ptr = other_names++;
+		while (*other_names != '|' && *other_names != '\0')
+		    other_names++;
+
+		if (*other_names != '\0')
+		    *(other_names++) = '\0';
+
+		key.data = ptr;
+		key.size = strlen(ptr);
+
+		_nc_db_put(capdb, &key, &data);
+	    }
+	    _nc_db_close(capdb);
+	}
+    }
+#else /* !USE_HASHED_DB */
+    if (call_count++ == 0) {
+	start_time = 0;
+    }
+
     if (strlen(first_name) > sizeof(filename) - 3)
 	_nc_warning("terminal name too long.");
 
@@ -351,20 +454,45 @@ _nc_write_entry(TERMTYPE * const tp)
 	    write_file(linkname, tp);
 #endif /* HAVE_LINK */
     }
+#endif /* USE_HASHED_DB */
 }
 
+static unsigned
+fake_write(char *dst,
+	   unsigned *offset,
+	   unsigned limit,
+	   char *src,
+	   unsigned want,
+	   unsigned size)
+{
+    int have = (limit - *offset);
+
+    want *= size;
+    if (have > 0) {
+	if ((int) want > have)
+	    want = have;
+	memcpy(dst + *offset, src, want);
+	*offset += want;
+    } else {
+	want = 0;
+    }
+    return (int) (want / size);
+}
+
+#define Write(buf, size, count) fake_write(buffer, offset, limit, (char *) buf, count, size)
+
 #undef LITTLE_ENDIAN		/* BSD/OS defines this as a feature macro */
 #define HI(x)			((x) / 256)
 #define LO(x)			((x) % 256)
 #define LITTLE_ENDIAN(p, x)	(p)[0] = LO(x), (p)[1] = HI(x)
 
-#define WRITE_STRING(str) (fwrite(str, sizeof(char), strlen(str) + 1, fp) == strlen(str) + 1)
+#define WRITE_STRING(str) (Write(str, sizeof(char), strlen(str) + 1) == strlen(str) + 1)
 
 static int
-compute_offsets(char **Strings, int strmax, short *offsets)
+compute_offsets(char **Strings, unsigned strmax, short *offsets)
 {
     size_t nextfree = 0;
-    int i;
+    unsigned i;
 
     for (i = 0; i < strmax; i++) {
 	if (Strings[i] == ABSENT_STRING) {
@@ -374,16 +502,17 @@ compute_offsets(char **Strings, int strmax, short *offsets)
 	} else {
 	    offsets[i] = nextfree;
 	    nextfree += strlen(Strings[i]) + 1;
-	    TRACE_OUT(("put Strings[%d]=%s(%d)", i, _nc_visbuf(Strings[i]), nextfree));
+	    TRACE_OUT(("put Strings[%d]=%s(%d)", (int) i,
+		       _nc_visbuf(Strings[i]), nextfree));
 	}
     }
     return nextfree;
 }
 
 static void
-convert_shorts(unsigned char *buf, short *Numbers, int count)
+convert_shorts(unsigned char *buf, short *Numbers, unsigned count)
 {
-    int i;
+    unsigned i;
     for (i = 0; i < count; i++) {
 	if (Numbers[i] == ABSENT_NUMERIC) {	/* HI/LO won't work */
 	    buf[2 * i] = buf[2 * i + 1] = 0377;
@@ -398,10 +527,68 @@ convert_shorts(unsigned char *buf, short *Numbers, int count)
 }
 
 #define even_boundary(value) \
-	    ((value) % 2 != 0 && fwrite(&zero, sizeof(char), 1, fp) != 1)
+	    ((value) % 2 != 0 && Write(&zero, sizeof(char), 1) != 1)
+
+#if NCURSES_XNAMES
+static unsigned
+extended_Booleans(TERMTYPE *tp)
+{
+    unsigned short result = 0;
+    unsigned short i;
+
+    for (i = 0; i < tp->ext_Booleans; ++i) {
+	if (tp->Booleans[BOOLCOUNT + i] == TRUE)
+	    result = (i + 1);
+    }
+    return result;
+}
+
+static unsigned
+extended_Numbers(TERMTYPE *tp)
+{
+    unsigned short result = 0;
+    unsigned short i;
+
+    for (i = 0; i < tp->ext_Numbers; ++i) {
+	if (tp->Numbers[NUMCOUNT + i] != ABSENT_NUMERIC)
+	    result = (i + 1);
+    }
+    return result;
+}
+
+static unsigned
+extended_Strings(TERMTYPE *tp)
+{
+    unsigned short result = 0;
+    unsigned short i;
+
+    for (i = 0; i < tp->ext_Strings; ++i) {
+	if (tp->Strings[STRCOUNT + i] != ABSENT_STRING)
+	    result = (i + 1);
+    }
+    return result;
+}
+
+/*
+ * _nc_align_termtype() will extend entries that are referenced in a use=
+ * clause - discard the unneeded data.
+ */
+static bool
+extended_object(TERMTYPE *tp)
+{
+    bool result = FALSE;
+
+    if (_nc_user_definable) {
+	result = ((extended_Booleans(tp)
+		   + extended_Numbers(tp)
+		   + extended_Strings(tp)) != 0);
+    }
+    return result;
+}
+#endif
 
 static int
-write_object(FILE * fp, TERMTYPE * tp)
+write_object(TERMTYPE *tp, char *buffer, unsigned *offset, unsigned limit)
 {
     char *namelist;
     size_t namelen, boolmax, nummax, strmax;
@@ -460,9 +647,9 @@ write_object(FILE * fp, TERMTYPE * tp)
     LITTLE_ENDIAN(buf + 10, nextfree);
 
     /* write out the header */
-    TRACE_OUT(("Header of %s @%ld", namelist, ftell(fp)));
-    if (fwrite(buf, 12, 1, fp) != 1
-	|| fwrite(namelist, sizeof(char), namelen, fp) != namelen)
+    TRACE_OUT(("Header of %s @%d", namelist, *offset));
+    if (Write(buf, 12, 1) != 1
+	|| Write(namelist, sizeof(char), namelen) != namelen)
 	  return (ERR);
 
     for (i = 0; i < boolmax; i++)
@@ -470,27 +657,27 @@ write_object(FILE * fp, TERMTYPE * tp)
 	    buf[i] = TRUE;
 	else
 	    buf[i] = FALSE;
-    if (fwrite(buf, sizeof(char), boolmax, fp) != boolmax)
+    if (Write(buf, sizeof(char), boolmax) != boolmax)
 	  return (ERR);
 
     if (even_boundary(namelen + boolmax))
 	return (ERR);
 
-    TRACE_OUT(("Numerics begin at %04lx", ftell(fp)));
+    TRACE_OUT(("Numerics begin at %04x", *offset));
 
     /* the numerics */
     convert_shorts(buf, tp->Numbers, nummax);
-    if (fwrite(buf, 2, nummax, fp) != nummax)
+    if (Write(buf, 2, nummax) != nummax)
 	return (ERR);
 
-    TRACE_OUT(("String offsets begin at %04lx", ftell(fp)));
+    TRACE_OUT(("String offsets begin at %04x", *offset));
 
     /* the string offsets */
     convert_shorts(buf, offsets, strmax);
-    if (fwrite(buf, 2, strmax, fp) != strmax)
+    if (Write(buf, 2, strmax) != strmax)
 	return (ERR);
 
-    TRACE_OUT(("String table begins at %04lx", ftell(fp)));
+    TRACE_OUT(("String table begins at %04x", *offset));
 
     /* the strings */
     for (i = 0; i < strmax; i++)
@@ -499,7 +686,7 @@ write_object(FILE * fp, TERMTYPE * tp)
 		return (ERR);
 
 #if NCURSES_XNAMES
-    if (NUM_EXT_NAMES(tp)) {
+    if (extended_object(tp)) {
 	unsigned extcnt = NUM_EXT_NAMES(tp);
 
 	if (even_boundary(nextfree))
@@ -519,23 +706,23 @@ write_object(FILE * fp, TERMTYPE * tp)
 	LITTLE_ENDIAN(buf + 4, tp->ext_Strings);
 	LITTLE_ENDIAN(buf + 6, strmax);
 	LITTLE_ENDIAN(buf + 8, nextfree);
-	TRACE_OUT(("WRITE extended-header @%ld", ftell(fp)));
-	if (fwrite(buf, 10, 1, fp) != 1)
+	TRACE_OUT(("WRITE extended-header @%d", *offset));
+	if (Write(buf, 10, 1) != 1)
 	    return (ERR);
 
-	TRACE_OUT(("WRITE %d booleans @%ld", tp->ext_Booleans, ftell(fp)));
+	TRACE_OUT(("WRITE %d booleans @%d", tp->ext_Booleans, *offset));
 	if (tp->ext_Booleans
-	    && fwrite(tp->Booleans + BOOLCOUNT, sizeof(char),
-		      tp->ext_Booleans, fp) != tp->ext_Booleans)
+	    && Write(tp->Booleans + BOOLCOUNT, sizeof(char),
+		     tp->ext_Booleans) != tp->ext_Booleans)
 	      return (ERR);
 
 	if (even_boundary(tp->ext_Booleans))
 	    return (ERR);
 
-	TRACE_OUT(("WRITE %d numbers @%ld", tp->ext_Numbers, ftell(fp)));
+	TRACE_OUT(("WRITE %d numbers @%d", tp->ext_Numbers, *offset));
 	if (tp->ext_Numbers) {
 	    convert_shorts(buf, tp->Numbers + NUMCOUNT, tp->ext_Numbers);
-	    if (fwrite(buf, 2, tp->ext_Numbers, fp) != tp->ext_Numbers)
+	    if (Write(buf, 2, tp->ext_Numbers) != tp->ext_Numbers)
 		return (ERR);
 	}
 
@@ -544,8 +731,8 @@ write_object(FILE * fp, TERMTYPE * tp)
 	 * in that order.
 	 */
 	convert_shorts(buf, offsets, strmax);
-	TRACE_OUT(("WRITE offsets @%ld", ftell(fp)));
-	if (fwrite(buf, 2, strmax, fp) != strmax)
+	TRACE_OUT(("WRITE offsets @%d", *offset));
+	if (Write(buf, 2, strmax) != strmax)
 	    return (ERR);
 
 	/*
@@ -554,7 +741,7 @@ write_object(FILE * fp, TERMTYPE * tp)
 	 */
 	for (i = 0; i < tp->ext_Strings; i++) {
 	    if (VALID_STRING(tp->Strings[i + STRCOUNT])) {
-		TRACE_OUT(("WRITE ext_Strings[%d]=%s", i,
+		TRACE_OUT(("WRITE ext_Strings[%d]=%s", (int) i,
 			   _nc_visbuf(tp->Strings[i + STRCOUNT])));
 		if (!WRITE_STRING(tp->Strings[i + STRCOUNT]))
 		    return (ERR);
@@ -565,7 +752,7 @@ write_object(FILE * fp, TERMTYPE * tp)
 	 * Write the extended names
 	 */
 	for (i = 0; i < extcnt; i++) {
-	    TRACE_OUT(("WRITE ext_Names[%d]=%s", i, tp->ext_Names[i]));
+	    TRACE_OUT(("WRITE ext_Names[%d]=%s", (int) i, tp->ext_Names[i]));
 	    if (!WRITE_STRING(tp->ext_Names[i]))
 		return (ERR);
 	}
