@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2001,2002 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2006,2007 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,10 +29,16 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Thomas E. Dickey                        1996-on                 *
  ****************************************************************************/
 
 /*
  *	lib_trace.c - Tracing/Debugging routines
+ *
+ * The _tracef() function is originally from pcurses (by Pavel Curtis) in 1982. 
+ * pcurses allowed one to enable/disable tracing using traceon() and traceoff()
+ * functions.  ncurses provides a trace() function which allows one to
+ * selectively enable or disable several tracing features.
  */
 
 #include <curses.priv.h>
@@ -40,29 +46,40 @@
 
 #include <ctype.h>
 
-MODULE_ID("$Id: lib_trace.c,v 1.50 2002/10/12 15:20:15 tom Exp $")
+MODULE_ID("$Id: lib_trace.c,v 1.62 2007/07/14 19:32:54 tom Exp $")
 
-NCURSES_EXPORT_VAR(unsigned) _nc_tracing = 0;	/* always define this */
+NCURSES_EXPORT_VAR(unsigned) _nc_tracing = 0; /* always define this */
 
 #ifdef TRACE
 NCURSES_EXPORT_VAR(const char *) _nc_tputs_trace = "";
 NCURSES_EXPORT_VAR(long) _nc_outchars = 0;
 
-static FILE *tracefp;		/* default to writing to stderr */
+#define TraceFP		_nc_globals.trace_fp
+#define TracePath	_nc_globals.trace_fname
+#define TraceLevel	_nc_globals.trace_level
 
 NCURSES_EXPORT(void)
-trace(const unsigned int tracelevel GCC_UNUSED)
+trace(const unsigned int tracelevel)
 {
-    static bool been_here = FALSE;
-    static char my_name[] = "trace";
+    if ((TraceFP == 0) && tracelevel) {
+	const char *mode = _nc_globals.init_trace ? "ab" : "wb";
 
-    if (!been_here && tracelevel) {
-	been_here = TRUE;
+	if (TracePath[0] == '\0') {
+	    if (getcwd(TracePath, sizeof(TracePath) - 12) == 0) {
+		perror("curses: Can't get working directory");
+		exit(EXIT_FAILURE);
+	    }
+	    strcat(TracePath, "/trace");
+	    if (_nc_is_dir_path(TracePath)) {
+		strcat(TracePath, ".log");
+	    }
+	}
 
+	_nc_globals.init_trace = TRUE;
 	_nc_tracing = tracelevel;
-	if (_nc_access(my_name, W_OK) < 0
-	    || (tracefp = fopen(my_name, "wb")) == 0) {
-	    perror("curses: Can't open 'trace' file: ");
+	if (_nc_access(TracePath, W_OK) < 0
+	    || (TraceFP = fopen(TracePath, mode)) == 0) {
+	    perror("curses: Can't open 'trace' file");
 	    exit(EXIT_FAILURE);
 	}
 	/* Try to set line-buffered mode, or (failing that) unbuffered,
@@ -70,12 +87,20 @@ trace(const unsigned int tracelevel GCC_UNUSED)
 	 * end of each line.  This is useful in case the program dies. 
 	 */
 #if HAVE_SETVBUF		/* ANSI */
-	(void) setvbuf(tracefp, (char *) 0, _IOLBF, 0);
+	(void) setvbuf(TraceFP, (char *) 0, _IOLBF, 0);
 #elif HAVE_SETBUF		/* POSIX */
-	(void) setbuffer(tracefp, (char *) 0);
+	(void) setbuffer(TraceFP, (char *) 0);
 #endif
-	_tracef("TRACING NCURSES version %s (tracelevel=%#x)",
-		curses_version(), tracelevel);
+	_tracef("TRACING NCURSES version %s.%d (tracelevel=%#x)",
+		NCURSES_VERSION,
+		NCURSES_VERSION_PATCH,
+		tracelevel);
+    } else if (tracelevel == 0) {
+	if (TraceFP != 0) {
+	    fclose(TraceFP);
+	    TraceFP = 0;
+	}
+	_nc_tracing = tracelevel;
     } else if (_nc_tracing != tracelevel) {
 	_nc_tracing = tracelevel;
 	_tracef("tracelevel=%#x", tracelevel);
@@ -87,22 +112,21 @@ _tracef(const char *fmt,...)
 {
     static const char Called[] = T_CALLED("");
     static const char Return[] = T_RETURN("");
-    static int level;
     va_list ap;
     bool before = FALSE;
     bool after = FALSE;
-    int doit = _nc_tracing;
+    unsigned doit = _nc_tracing;
     int save_err = errno;
 
     if (strlen(fmt) >= sizeof(Called) - 1) {
 	if (!strncmp(fmt, Called, sizeof(Called) - 1)) {
 	    before = TRUE;
-	    level++;
+	    TraceLevel++;
 	} else if (!strncmp(fmt, Return, sizeof(Return) - 1)) {
 	    after = TRUE;
 	}
 	if (before || after) {
-	    if ((level <= 1)
+	    if ((TraceLevel <= 1)
 		|| (doit & TRACE_ICALLS) != 0)
 		doit &= (TRACE_CALLS | TRACE_CCALLS);
 	    else
@@ -111,22 +135,22 @@ _tracef(const char *fmt,...)
     }
 
     if (doit != 0) {
-	if (tracefp == 0)
-	    tracefp = stderr;
+	if (TraceFP == 0)
+	    TraceFP = stderr;
 	if (before || after) {
 	    int n;
-	    for (n = 1; n < level; n++)
-		fputs("+ ", tracefp);
+	    for (n = 1; n < TraceLevel; n++)
+		fputs("+ ", TraceFP);
 	}
 	va_start(ap, fmt);
-	vfprintf(tracefp, fmt, ap);
-	fputc('\n', tracefp);
+	vfprintf(TraceFP, fmt, ap);
+	fputc('\n', TraceFP);
 	va_end(ap);
-	fflush(tracefp);
+	fflush(TraceFP);
     }
 
-    if (after && level)
-	level--;
+    if (after && TraceLevel)
+	TraceLevel--;
     errno = save_err;
 }
 
@@ -162,9 +186,33 @@ _nc_retrace_ptr(char *code)
     return code;
 }
 
+/* Trace 'const char*' return-values */
+NCURSES_EXPORT(const char *)
+_nc_retrace_cptr(const char *code)
+{
+    T((T_RETURN("%s"), _nc_visbuf(code)));
+    return code;
+}
+
+/* Trace 'NCURSES_CONST void*' return-values */
+NCURSES_EXPORT(NCURSES_CONST void *)
+_nc_retrace_cvoid_ptr(NCURSES_CONST void *code)
+{
+    T((T_RETURN("%p"), code));
+    return code;
+}
+
+/* Trace 'void*' return-values */
+NCURSES_EXPORT(void *)
+_nc_retrace_void_ptr(void *code)
+{
+    T((T_RETURN("%p"), code));
+    return code;
+}
+
 /* Trace 'SCREEN *' return-values */
 NCURSES_EXPORT(SCREEN *)
-_nc_retrace_sp(SCREEN * code)
+_nc_retrace_sp(SCREEN *code)
 {
     T((T_RETURN("%p"), code));
     return code;
