@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2005,2006 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,6 +29,7 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Thomas E. Dickey                        1996 on                 *
  ****************************************************************************/
 
 /*
@@ -64,7 +65,7 @@
 #include <curses.priv.h>
 #include <term.h>
 
-MODULE_ID("$Id: lib_vidattr.c,v 1.33 2000/10/09 22:45:29 tom Exp $")
+MODULE_ID("$Id: lib_vidattr.c,v 1.46 2006/01/21 23:39:40 tom Exp $")
 
 #define doPut(mode) TPUTS_TRACE(#mode); tputs(mode, 1, outc)
 
@@ -86,14 +87,13 @@ MODULE_ID("$Id: lib_vidattr.c,v 1.33 2000/10/09 22:45:29 tom Exp $")
 		} \
 	}
 
-int
-vidputs(attr_t newmode, int (*outc) (int))
+NCURSES_EXPORT(int)
+vidputs(chtype newmode, int (*outc) (int))
 {
     static attr_t previous_attr = A_NORMAL;
     attr_t turn_on, turn_off;
     int pair;
     bool reverse = FALSE;
-    bool used_ncv = FALSE;
     bool can_color = (SP == 0 || SP->_coloron);
 #if NCURSES_EXT_FUNCS
     bool fix_pair0 = (SP != 0 && SP->_coloron && !SP->_default_color);
@@ -105,15 +105,51 @@ vidputs(attr_t newmode, int (*outc) (int))
 
     /* this allows us to go on whether or not newterm() has been called */
     if (SP)
-	previous_attr = SP->_current_attr;
+	previous_attr = AttrOf(SCREEN_ATTRS(SP));
 
     TR(TRACE_ATTRS, ("previous attribute was %s", _traceattr(previous_attr)));
 
-#if !USE_XMC_SUPPORT
     if ((SP != 0)
-	&& (magic_cookie_glitch > 0))
+	&& (magic_cookie_glitch > 0)) {
+#if USE_XMC_SUPPORT
+	static chtype table[] =
+	{
+	    A_STANDOUT,
+	    A_UNDERLINE,
+	    A_REVERSE,
+	    A_BLINK,
+	    A_DIM,
+	    A_BOLD,
+	    A_INVIS,
+	    A_PROTECT,
+	};
+	unsigned n;
+	int used = 0;
+	int limit = (max_attributes <= 0) ? 1 : max_attributes;
+	chtype retain = 0;
+
+	/*
+	 * Limit the number of attribute bits set in the newmode according to
+	 * the terminfo max_attributes value.
+	 */
+	for (n = 0; n < SIZEOF(table); ++n) {
+	    if ((table[n] & SP->_ok_attributes) == 0) {
+		newmode &= ~table[n];
+	    } else if ((table[n] & newmode) != 0) {
+		if (used++ >= limit) {
+		    newmode &= ~table[n];
+		    if (newmode == retain)
+			break;
+		} else {
+		    retain = newmode;
+		}
+	    }
+	}
+#else
 	newmode &= ~(SP->_xmc_suppress);
 #endif
+	TR(TRACE_ATTRS, ("suppressed attribute is %s", _traceattr(newmode)));
+    }
 
     /*
      * If we have a terminal that cannot combine color with video
@@ -134,11 +170,13 @@ vidputs(attr_t newmode, int (*outc) (int))
 	 * A_ALTCHARSET (256) down 2 to line up.  We use the NCURSES_BITS
 	 * macro so this will work properly for the wide-character layout.
 	 */
-	attr_t mask = NCURSES_BITS((no_color_video & 63)
-				   | ((no_color_video & 192) << 1)
-				   | ((no_color_video & 256) >> 2), 8);
+	unsigned value = no_color_video;
+	attr_t mask = NCURSES_BITS((value & 63)
+				   | ((value & 192) << 1)
+				   | ((value & 256) >> 2), 8);
 
-	if (mask & A_REVERSE && newmode & A_REVERSE) {
+	if ((mask & A_REVERSE) != 0
+	    && (newmode & A_REVERSE) != 0) {
 	    reverse = TRUE;
 	    mask &= ~A_REVERSE;
 	}
@@ -175,11 +213,11 @@ vidputs(attr_t newmode, int (*outc) (int))
 		    TurnOff(A_STANDOUT, exit_standout_mode);
 		}
 	    }
-	    previous_attr &= ~A_COLOR;
+	    previous_attr &= ALL_BUT_COLOR;
 	}
 
 	SetColorsIf((pair != 0) || fix_pair0, previous_attr);
-    } else if (set_attributes && !used_ncv) {
+    } else if (set_attributes) {
 	if (turn_on || turn_off) {
 	    TPUTS_TRACE("set_attributes");
 	    tputs(tparm(set_attributes,
@@ -192,7 +230,7 @@ vidputs(attr_t newmode, int (*outc) (int))
 			(newmode & A_INVIS) != 0,
 			(newmode & A_PROTECT) != 0,
 			(newmode & A_ALTCHARSET) != 0), 1, outc);
-	    previous_attr &= ~A_COLOR;
+	    previous_attr &= ALL_BUT_COLOR;
 	}
 	SetColorsIf((pair != 0) || fix_pair0, previous_attr);
     } else {
@@ -211,8 +249,8 @@ vidputs(attr_t newmode, int (*outc) (int))
 
 	if (turn_off && exit_attribute_mode) {
 	    doPut(exit_attribute_mode);
-	    turn_on |= (newmode & (chtype) (~A_COLOR));
-	    previous_attr &= ~A_COLOR;
+	    turn_on |= (newmode & ALL_BUT_COLOR);
+	    previous_attr &= ALL_BUT_COLOR;
 	}
 	SetColorsIf((pair != 0) || fix_pair0, previous_attr);
 
@@ -227,12 +265,14 @@ vidputs(attr_t newmode, int (*outc) (int))
 	TurnOn(A_PROTECT,	enter_protected_mode);
 	TurnOn(A_INVIS,		enter_secure_mode);
 	TurnOn(A_UNDERLINE,	enter_underline_mode);
+#if USE_WIDEC_SUPPORT
 	TurnOn(A_HORIZONTAL,	enter_horizontal_hl_mode);
 	TurnOn(A_LEFT,		enter_left_hl_mode);
 	TurnOn(A_LOW,		enter_low_hl_mode);
 	TurnOn(A_RIGHT,		enter_right_hl_mode);
 	TurnOn(A_TOP,		enter_top_hl_mode);
 	TurnOn(A_VERTICAL,	enter_vertical_hl_mode);
+#endif
 	/* *INDENT-ON* */
 
     }
@@ -241,22 +281,22 @@ vidputs(attr_t newmode, int (*outc) (int))
 	newmode |= A_REVERSE;
 
     if (SP)
-	SP->_current_attr = newmode;
+	SetAttr(SCREEN_ATTRS(SP), newmode);
     else
 	previous_attr = newmode;
 
     returnCode(OK);
 }
 
-int
-vidattr(attr_t newmode)
+NCURSES_EXPORT(int)
+vidattr(chtype newmode)
 {
     T((T_CALLED("vidattr(%s)"), _traceattr(newmode)));
 
     returnCode(vidputs(newmode, _nc_outch));
 }
 
-chtype
+NCURSES_EXPORT(chtype)
 termattrs(void)
 {
     chtype attrs = A_NORMAL;
