@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2003,2004 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2006,2007 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,7 +29,7 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
- *     and: Thomas E. Dickey 1996-2004                                      *
+ *     and: Thomas E. Dickey                        1996-on                 *
  ****************************************************************************/
 
 /*-----------------------------------------------------------------
@@ -74,7 +74,7 @@
 #include <ctype.h>
 #include <term.h>
 
-MODULE_ID("$Id: tty_update.c,v 1.205 2004/02/07 18:02:42 tom Exp $")
+MODULE_ID("$Id: tty_update.c,v 1.240 2007/06/30 21:07:51 tom Exp $")
 
 /*
  * This define controls the line-breakout optimization.  Every once in a
@@ -89,6 +89,9 @@ MODULE_ID("$Id: tty_update.c,v 1.205 2004/02/07 18:02:42 tom Exp $")
 
 #define FILL_BCE() (SP->_coloron && !SP->_default_color && !back_color_erase)
 
+static const NCURSES_CH_T blankchar = NewChar(BLANK_TEXT);
+static NCURSES_CH_T normal = NewChar(BLANK_TEXT);
+
 /*
  * Enable checking to see if doupdate and friends are tracking the true
  * cursor position correctly.  NOTE: this is a debugging hack which will
@@ -96,7 +99,7 @@ MODULE_ID("$Id: tty_update.c,v 1.205 2004/02/07 18:02:42 tom Exp $")
  */
 /* #define POSITION_DEBUG */
 
-static inline NCURSES_CH_T ClrBlank(WINDOW *win);
+static NCURSES_INLINE NCURSES_CH_T ClrBlank(WINDOW *win);
 static int ClrBottom(int total);
 static void ClearScreen(NCURSES_CH_T blank);
 static void ClrUpdate(void);
@@ -164,7 +167,7 @@ position_check(int expected_y, int expected_x, char *legend)
  *
  ****************************************************************************/
 
-static inline void
+static NCURSES_INLINE void
 GoTo(int const row, int const col)
 {
     TR(TRACE_MOVE, ("GoTo(%d, %d) from (%d, %d)",
@@ -176,14 +179,14 @@ GoTo(int const row, int const col)
     position_check(SP->_cursrow, SP->_curscol, "GoTo2");
 }
 
-static inline void
+static NCURSES_INLINE void
 PutAttrChar(CARG_CH_T ch)
 {
     int chlen = 1;
     NCURSES_CH_T my_ch;
     PUTC_DATA;
     NCURSES_CH_T tilde;
-    NCURSES_ATTR_T attr = AttrOfD(ch);
+    NCURSES_CH_T attr = CHDEREF(ch);
 
     TR(TRACE_CHARPUT, ("PutAttrChar(%s) at (%d, %d)",
 		       _tracech_t(ch),
@@ -192,17 +195,38 @@ PutAttrChar(CARG_CH_T ch)
     /*
      * If this is not a valid character, there is nothing more to do.
      */
-    if (isnac(CHDEREF(ch)))
+    if (isWidecExt(CHDEREF(ch))) {
+	TR(TRACE_CHARPUT, ("...skip"));
 	return;
+    }
     /*
      * Determine the number of character cells which the 'ch' value will use
      * on the screen.  It should be at least one.
      */
     if ((chlen = wcwidth(CharOf(CHDEREF(ch)))) <= 0) {
-	static NCURSES_CH_T blank = NewChar(BLANK_TEXT);
+	static const NCURSES_CH_T blank = NewChar(BLANK_TEXT);
 
-	if (isprint(CharOf(CHDEREF(ch)))
-	    || (SP->_posix_locale && CharOf(CHDEREF(ch)) >= 160)) {
+	/*
+	 * If the character falls into any of these special cases, do
+	 * not force the result to a blank:
+	 *
+	 * a) it is printable (this works around a bug in wcwidth()).
+	 * b) use_legacy_coding() has been called to modify the treatment
+	 *    of codes 128-255.
+	 * c) the acs_map[] has been initialized to allow codes 0-31
+	 *    to be rendered.  This supports Linux console's "PC"
+	 *    characters.  Codes 128-255 are allowed though this is
+	 *    not checked.
+	 */
+	if (is8bits(CharOf(CHDEREF(ch)))
+	    && (isprint(CharOf(CHDEREF(ch)))
+		|| (SP->_legacy_coding > 0 && CharOf(CHDEREF(ch)) >= 160)
+		|| (SP->_legacy_coding > 1 && CharOf(CHDEREF(ch)) >= 128)
+		|| (AttrOf(attr) & A_ALTCHARSET
+		    && ((CharOfD(ch) < ACS_LEN
+			 && SP->_acs_map != 0
+			 && SP->_acs_map[CharOfD(ch)] != 0)
+			|| (CharOfD(ch) >= 128))))) {
 	    ;
 	} else {
 	    ch = CHREF(blank);
@@ -212,7 +236,7 @@ PutAttrChar(CARG_CH_T ch)
     }
 #endif
 
-    if ((attr & A_ALTCHARSET)
+    if ((AttrOf(attr) & A_ALTCHARSET)
 	&& SP->_acs_map != 0
 	&& CharOfD(ch) < ACS_LEN) {
 	my_ch = CHDEREF(ch);	/* work around const param */
@@ -224,21 +248,33 @@ PutAttrChar(CARG_CH_T ch)
 	 * normal one to be broken (by mis-design ;-).
 	 */
 	if (SP->_screen_acs_fix
-	    && SP->_acs_map[CharOf(my_ch)] & A_ALTCHARSET) {
-	    attr &= ~(A_ALTCHARSET);
+	    && SP->_screen_acs_map[CharOf(my_ch)]) {
+	    RemAttr(attr, A_ALTCHARSET);
 	    my_ch = _nc_wacs[CharOf(my_ch)];
 	}
 #endif
-	if (attr & A_ALTCHARSET) {
-	    chtype temp = UChar(SP->_acs_map[CharOfD(ch)]);
+	/*
+	 * If we (still) have alternate character set, it is the normal 8bit
+	 * flavor.  The _screen_acs_map[] array tells if the character was
+	 * really in acs_chars, needed because of the way wide/normal line
+	 * drawing flavors are integrated.
+	 */
+	if (AttrOf(attr) & A_ALTCHARSET) {
+	    int j = CharOfD(ch);
+	    chtype temp = UChar(SP->_acs_map[j]);
+
+	    if (!(SP->_screen_acs_map[j])) {
+		RemAttr(attr, A_ALTCHARSET);
+		if (temp == 0)
+		    temp = ' ';
+	    }
 	    if (temp != 0)
-		SetChar(my_ch, temp, attr);
-	    RemAttr(my_ch, A_ALTCHARSET);
+		SetChar(my_ch, temp, AttrOf(attr));
 	}
 	ch = CHREF(my_ch);
     }
     if (tilde_glitch && (CharOfD(ch) == L('~'))) {
-	SetChar(tilde, L('`'), attr);
+	SetChar(tilde, L('`'), AttrOf(attr));
 	ch = CHREF(tilde);
     }
 
@@ -347,38 +383,48 @@ PutCharLR(const ARG_CH_T ch)
     }
 }
 
+/*
+ * Wrap the cursor position, i.e., advance to the beginning of the next line.
+ */
 static void
 wrap_cursor(void)
 {
     if (eat_newline_glitch) {
 	/*
-	 * xenl can manifest two different ways.  The vt100
-	 * way is that, when you'd expect the cursor to wrap,
-	 * it stays hung at the right margin (on top of the
-	 * character just emitted) and doesn't wrap until the
-	 * *next* graphic char is emitted.  The c100 way is
-	 * to ignore LF received just after an am wrap.
+	 * xenl can manifest two different ways.  The vt100 way is that, when
+	 * you'd expect the cursor to wrap, it stays hung at the right margin
+	 * (on top of the character just emitted) and doesn't wrap until the
+	 * *next* graphic char is emitted.  The c100 way is to ignore LF
+	 * received just after an am wrap.
 	 *
-	 * An aggressive way to handle this would be to
-	 * emit CR/LF after the char and then assume the wrap
-	 * is done, you're on the first position of the next
-	 * line, and the terminal out of its weird state.
-	 * Here it's safe to just tell the code that the
-	 * cursor is in hyperspace and let the next mvcur()
-	 * call straighten things out.
+	 * An aggressive way to handle this would be to emit CR/LF after the
+	 * char and then assume the wrap is done, you're on the first position
+	 * of the next line, and the terminal out of its weird state.  Here
+	 * it's safe to just tell the code that the cursor is in hyperspace and
+	 * let the next mvcur() call straighten things out.
 	 */
 	SP->_curscol = -1;
 	SP->_cursrow = -1;
     } else if (auto_right_margin) {
 	SP->_curscol = 0;
 	SP->_cursrow++;
+	/*
+	 * We've actually moved - but may have to work around problems with
+	 * video attributes not working.
+	 */
+	if (!move_standout_mode && AttrOf(SCREEN_ATTRS(SP))) {
+	    TR(TRACE_CHARPUT, ("turning off (%#lx) %s before wrapping",
+			       (unsigned long) AttrOf(SCREEN_ATTRS(SP)),
+			       _traceattr(AttrOf(SCREEN_ATTRS(SP)))));
+	    (void) VIDATTR(A_NORMAL, 0);
+	}
     } else {
 	SP->_curscol--;
     }
     position_check(SP->_cursrow, SP->_curscol, "wrap_cursor");
 }
 
-static inline void
+static NCURSES_INLINE void
 PutChar(const ARG_CH_T ch)
 /* insert character, handling automargin stuff */
 {
@@ -399,18 +445,20 @@ PutChar(const ARG_CH_T ch)
  * as A_REVERSE.  All attribute flags which don't affect appearance of a space
  * or can be output by clearing (A_COLOR in case of bce-terminal) are excluded.
  */
-static inline bool
+static NCURSES_INLINE bool
 can_clear_with(ARG_CH_T ch)
 {
     if (!back_color_erase && SP->_coloron) {
 #if NCURSES_EXT_FUNCS
+	int pair;
+
 	if (!SP->_default_color)
 	    return FALSE;
 	if (SP->_default_fg != C_MASK || SP->_default_bg != C_MASK)
 	    return FALSE;
-	if (AttrOfD(ch) & A_COLOR) {
+	if ((pair = GetPair(CHDEREF(ch))) != 0) {
 	    short fg, bg;
-	    pair_content(PAIR_NUMBER(AttrOfD(ch)), &fg, &bg);
+	    pair_content(pair, &fg, &bg);
 	    if (fg != C_MASK || bg != C_MASK)
 		return FALSE;
 	}
@@ -440,6 +488,8 @@ EmitRange(const NCURSES_CH_T * ntext, int num)
 {
     int i;
 
+    TR(TRACE_CHARPUT, ("EmitRange %d:%s", num, _nc_viscbuf(ntext, num)));
+
     if (erase_chars || repeat_char) {
 	while (num > 0) {
 	    int runcount;
@@ -471,8 +521,8 @@ EmitRange(const NCURSES_CH_T * ntext, int num)
 	    if (erase_chars
 		&& runcount > SP->_ech_cost + SP->_cup_ch_cost
 		&& can_clear_with(CHREF(ntext0))) {
-		UpdateAttrs(AttrOf(ntext0));
-		putp(tparm(erase_chars, runcount));
+		UpdateAttrs(ntext0);
+		putp(TPARM_1(erase_chars, runcount));
 
 		/*
 		 * If this is the last part of the given interval,
@@ -491,8 +541,8 @@ EmitRange(const NCURSES_CH_T * ntext, int num)
 		if (wrap_possible)
 		    rep_count--;
 
-		UpdateAttrs(AttrOf(ntext0));
-		tputs(tparm(repeat_char, CharOf(ntext0), rep_count),
+		UpdateAttrs(ntext0);
+		tputs(TPARM_2(repeat_char, CharOf(ntext0), rep_count),
 		      rep_count, _nc_outch);
 		SP->_curscol += rep_count;
 
@@ -535,7 +585,7 @@ PutRange(const NCURSES_CH_T * otext,
     if (otext != ntext
 	&& (last - first + 1) > SP->_inline_cost) {
 	for (j = first, same = 0; j <= last; j++) {
-	    if (!same && isnac(otext[j]))
+	    if (!same && isWidecExt(otext[j]))
 		continue;
 	    if (CharEq(otext[j], ntext[j])) {
 		same++;
@@ -590,7 +640,7 @@ doupdate(void)
 	SP->_fifohold--;
 
 #if USE_SIZECHANGE
-    if (SP->_endwin || SP->_sig_winch) {
+    if (SP->_endwin || _nc_handle_sigwinch(FALSE)) {
 	/*
 	 * This is a transparent extension:  XSI does not address it,
 	 * and applications need not know that ncurses can do it.
@@ -621,17 +671,15 @@ doupdate(void)
 #endif /* USE_TRACE_TIMES */
 
     /*
-     * This is the support for magic-cookie terminals.  The
-     * theory: we scan the virtual screen looking for attribute
-     * turnons.  Where we find one, check to make sure it's
-     * realizable by seeing if the required number of
-     * un-attributed blanks are present before and after the
-     * attributed range; try to shift the range boundaries over
-     * blanks (not changing the screen display) so this becomes
-     * true.  If it is, shift the beginning attribute change
-     * appropriately (the end one, if we've gotten this far, is
-     * guaranteed room for its cookie). If not, nuke the added
-     * attributes out of the span.
+     * This is the support for magic-cookie terminals.  The theory:  we scan
+     * the virtual screen looking for attribute turnons.  Where we find one,
+     * check to make sure it's realizable by seeing if the required number of
+     * un-attributed blanks are present before and after the attributed range;
+     * try to shift the range boundaries over blanks (not changing the screen
+     * display) so this becomes true.  If it is, shift the beginning attribute
+     * change appropriately (the end one, if we've gotten this far, is
+     * guaranteed room for its cookie).  If not, nuke the added attributes out
+     * of the span.
      */
 #if USE_XMC_SUPPORT
     if (magic_cookie_glitch > 0) {
@@ -641,11 +689,13 @@ doupdate(void)
 	for (i = 0; i < screen_lines; i++) {
 	    for (j = 0; j < screen_columns; j++) {
 		bool failed = FALSE;
-		attr_t turnon = AttrOf(newscr->_line[i].text[j]) & ~rattr;
+		NCURSES_CH_T *thisline = newscr->_line[i].text;
+		attr_t thisattr = AttrOf(thisline[j]) & SP->_xmc_triggers;
+		attr_t turnon = thisattr & ~rattr;
 
 		/* is an attribute turned on here? */
 		if (turnon == 0) {
-		    rattr = AttrOf(newscr->_line[i].text[j]);
+		    rattr = thisattr;
 		    continue;
 		}
 
@@ -653,23 +703,33 @@ doupdate(void)
 		TR(TRACE_ATTRS, ("...to %s", _traceattr(turnon)));
 
 		/*
-		 * If the attribute change location is a blank with a
-		 * "safe" attribute, undo the attribute turnon.  This may
-		 * ensure there's enough room to set the attribute before
-		 * the first non-blank in the run.
+		 * If the attribute change location is a blank with a "safe"
+		 * attribute, undo the attribute turnon.  This may ensure
+		 * there's enough room to set the attribute before the first
+		 * non-blank in the run.
 		 */
-#define SAFE(a)	(!((a) & (attr_t)~NONBLANK_ATTR))
-		if (ISBLANK(newscr->_line[i].text[j]) && SAFE(turnon)) {
-		    RemAttr(newscr->_line[i].text[j], turnon);
+#define SAFE(a)	(!((a) & SP->_xmc_triggers))
+		if (ISBLANK(thisline[j]) && SAFE(turnon)) {
+		    RemAttr(thisline[j], turnon);
 		    continue;
 		}
 
 		/* check that there's enough room at start of span */
 		for (k = 1; k <= magic_cookie_glitch; k++) {
 		    if (j - k < 0
-			|| !ISBLANK(newscr->_line[i].text[j - k])
-			|| !SAFE(AttrOf(newscr->_line[i].text[j - k])))
+			|| !ISBLANK(thisline[j - k])
+			|| !SAFE(AttrOf(thisline[j - k]))) {
 			failed = TRUE;
+			TR(TRACE_ATTRS, ("No room at start in %d,%d%s%s",
+					 i, j - k,
+					 (ISBLANK(thisline[j - k])
+					  ? ""
+					  : ":nonblank"),
+					 (SAFE(AttrOf(thisline[j - k]))
+					  ? ""
+					  : ":unsafe")));
+			break;
+		    }
 		}
 		if (!failed) {
 		    bool end_onscreen = FALSE;
@@ -678,7 +738,8 @@ doupdate(void)
 		    /* find end of span, if it's onscreen */
 		    for (m = i; m < screen_lines; m++) {
 			for (; n < screen_columns; n++) {
-			    if (AttrOf(newscr->_line[m].text[n]) == rattr) {
+			    attr_t testattr = AttrOf(newscr->_line[m].text[n]);
+			    if ((testattr & SP->_xmc_triggers) == rattr) {
 				end_onscreen = TRUE;
 				TR(TRACE_ATTRS,
 				   ("Range attributed with %s ends at (%d, %d)",
@@ -697,22 +758,34 @@ doupdate(void)
 			NCURSES_CH_T *lastline = newscr->_line[m].text;
 
 			/*
-			 * If there are safely-attributed blanks at the
-			 * end of the range, shorten the range.  This will
-			 * help ensure that there is enough room at end
-			 * of span.
+			 * If there are safely-attributed blanks at the end of
+			 * the range, shorten the range.  This will help ensure
+			 * that there is enough room at end of span.
 			 */
 			while (n >= 0
 			       && ISBLANK(lastline[n])
-			       && SAFE(AttrOf(lastline[n])))
+			       && SAFE(AttrOf(lastline[n]))) {
 			    RemAttr(lastline[n--], turnon);
+			}
 
 			/* check that there's enough room at end of span */
-			for (k = 1; k <= magic_cookie_glitch; k++)
+			for (k = 1; k <= magic_cookie_glitch; k++) {
 			    if (n + k >= screen_columns
 				|| !ISBLANK(lastline[n + k])
-				|| !SAFE(AttrOf(lastline[n + k])))
+				|| !SAFE(AttrOf(lastline[n + k]))) {
 				failed = TRUE;
+				TR(TRACE_ATTRS,
+				   ("No room at end in %d,%d%s%s",
+				    i, j - k,
+				    (ISBLANK(lastline[n + k])
+				     ? ""
+				     : ":nonblank"),
+				    (SAFE(AttrOf(lastline[n + k]))
+				     ? ""
+				     : ":unsafe")));
+				break;
+			    }
+			}
 		    }
 		}
 
@@ -726,7 +799,8 @@ doupdate(void)
 		    /* turn off new attributes over span */
 		    for (p = i; p < screen_lines; p++) {
 			for (; q < screen_columns; q++) {
-			    if (AttrOf(newscr->_line[p].text[q]) == rattr)
+			    attr_t testattr = AttrOf(newscr->_line[p].text[q]);
+			    if ((testattr & SP->_xmc_triggers) == rattr)
 				goto foundend;
 			    RemAttr(newscr->_line[p].text[q], turnon);
 			}
@@ -739,14 +813,14 @@ doupdate(void)
 			_traceattr(turnon), i, j));
 
 		    /*
-		     * back up the start of range so there's room
-		     * for cookies before the first nonblank character
+		     * Back up the start of range so there's room for cookies
+		     * before the first nonblank character.
 		     */
 		    for (k = 1; k <= magic_cookie_glitch; k++)
-			AddAttr(newscr->_line[i].text[j - k], turnon);
+			AddAttr(thisline[j - k], turnon);
 		}
 
-		rattr = AttrOf(newscr->_line[i].text[j]);
+		rattr = thisattr;
 	    }
 	}
 
@@ -762,7 +836,6 @@ doupdate(void)
 
     nonempty = 0;
     if (curscr->_clear || newscr->_clear) {	/* force refresh ? */
-	TR(TRACE_UPDATE, ("clearing and updating from scratch"));
 	ClrUpdate();
 	curscr->_clear = FALSE;	/* reset flag */
 	newscr->_clear = FALSE;	/* reset flag */
@@ -830,13 +903,18 @@ doupdate(void)
 
   cleanup:
     /*
-     * Keep the physical screen in normal mode in case we get other
-     * processes writing to the screen.
+     * We would like to keep the physical screen in normal mode in case we get
+     * other processes writing to the screen.  This goal cannot be met for
+     * magic cookies since it interferes with attributes that may propagate
+     * past the current position.
      */
-    UpdateAttrs(A_NORMAL);
+#if USE_XMC_SUPPORT
+    if (magic_cookie_glitch != 0)
+#endif
+	UpdateAttrs(normal);
 
     _nc_flush();
-    curscr->_attrs = newscr->_attrs;
+    WINDOW_ATTRS(curscr) = WINDOW_ATTRS(newscr);
 
 #if USE_TRACE_TIMES
     (void) times(&after);
@@ -865,10 +943,10 @@ doupdate(void)
 #define BCE_ATTRS (A_NORMAL|A_COLOR)
 #define BCE_BKGD(win) (((win) == curscr ? stdscr : (win))->_nc_bkgd)
 
-static inline NCURSES_CH_T
+static NCURSES_INLINE NCURSES_CH_T
 ClrBlank(WINDOW *win)
 {
-    NCURSES_CH_T blank = NewChar(BLANK_TEXT);
+    NCURSES_CH_T blank = blankchar;
     if (back_color_erase)
 	AddAttr(blank, (AttrOf(BCE_BKGD(win)) & BCE_ATTRS));
     return blank;
@@ -888,7 +966,7 @@ ClrUpdate(void)
     NCURSES_CH_T blank = ClrBlank(stdscr);
     int nonempty = min(screen_lines, newscr->_maxy + 1);
 
-    TR(TRACE_UPDATE, ("ClrUpdate() called"));
+    TR(TRACE_UPDATE, (T_CALLED("ClrUpdate")));
 
     ClearScreen(blank);
 
@@ -898,6 +976,8 @@ ClrUpdate(void)
 
     for (i = 0; i < nonempty; i++)
 	TransformLine(i);
+
+    TR(TRACE_UPDATE, (T_RETURN("")));
 }
 
 /*
@@ -928,7 +1008,7 @@ ClrToEOL(NCURSES_CH_T blank, bool needclear)
     }
 
     if (needclear) {
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	TPUTS_TRACE("clr_eol");
 	if (clr_eol && SP->_el_cost <= (screen_columns - SP->_curscol)) {
 	    putp(clr_eol);
@@ -954,7 +1034,7 @@ ClrToEOS(NCURSES_CH_T blank)
     row = SP->_cursrow;
     col = SP->_curscol;
 
-    UpdateAttrs(AttrOf(blank));
+    UpdateAttrs(blank);
     TPUTS_TRACE("clr_eos");
     tputs(clr_eos, screen_lines - row, _nc_outch);
 
@@ -1015,14 +1095,8 @@ ClrBottom(int total)
 
 #if USE_XMC_SUPPORT
 #if USE_WIDEC_SUPPORT
-static inline bool
-check_xmc_transition(NCURSES_CH_T * a, NCURSES_CH_T * b)
-{
-    if (((a->attr ^ b->attr) & ~(a->attr) & SP->_xmc_triggers) != 0) {
-	return TRUE;
-    }
-    return FALSE;
-}
+#define check_xmc_transition(a, b) \
+    ((((a)->attr ^ (b)->attr) & ~((a)->attr) & SP->_xmc_triggers) != 0)
 #define xmc_turn_on(a,b) check_xmc_transition(&(a), &(b))
 #else
 #define xmc_turn_on(a,b) ((((a)^(b)) & ~(a) & SP->_xmc_triggers) != 0)
@@ -1059,14 +1133,12 @@ TransformLine(int const lineno)
     int n;
     bool attrchanged = FALSE;
 
-    TR(TRACE_UPDATE, ("TransformLine(%d) called", lineno));
+    TR(TRACE_UPDATE, (T_CALLED("TransformLine(%d)"), lineno));
 
     /* copy new hash value to old one */
     if (SP->oldhash && SP->newhash)
 	SP->oldhash[lineno] = SP->newhash[lineno];
 
-#define ColorOf(n) (AttrOf(n) & A_COLOR)
-#define unColor(n) (AttrOf(n) & ALL_BUT_COLOR)
     /*
      * If we have colors, there is the possibility of having two color pairs
      * that display as the same colors.  For instance, Lynx does this.  Check
@@ -1074,24 +1146,19 @@ TransformLine(int const lineno)
      * they are equivalent.
      */
     if (SP->_coloron) {
-	attr_t oldColor;
-	attr_t newColor;
 	int oldPair;
 	int newPair;
 
 	for (n = 0; n < screen_columns; n++) {
 	    if (!CharEq(newLine[n], oldLine[n])) {
-		oldColor = ColorOf(oldLine[n]);
-		newColor = ColorOf(newLine[n]);
-		if (oldColor != newColor
+		oldPair = GetPair(oldLine[n]);
+		newPair = GetPair(newLine[n]);
+		if (oldPair != newPair
 		    && unColor(oldLine[n]) == unColor(newLine[n])) {
-		    oldPair = PAIR_NUMBER(oldColor);
-		    newPair = PAIR_NUMBER(newColor);
 		    if (oldPair < COLOR_PAIRS
 			&& newPair < COLOR_PAIRS
 			&& SP->_color_pairs[oldPair] == SP->_color_pairs[newPair]) {
-			RemAttr(oldLine[n], A_COLOR);
-			AddAttr(oldLine[n], ColorOf(newLine[n]));
+			SetPair(oldLine[n], GetPair(newLine[n]));
 		    }
 		}
 	    }
@@ -1101,7 +1168,7 @@ TransformLine(int const lineno)
     if (ceol_standout_glitch && clr_eol) {
 	firstChar = 0;
 	while (firstChar < screen_columns) {
-	    if (AttrOf(newLine[firstChar]) != AttrOf(oldLine[firstChar])) {
+	    if (!SameAttrOf(newLine[firstChar], oldLine[firstChar])) {
 		attrchanged = TRUE;
 		break;
 	    }
@@ -1193,12 +1260,12 @@ TransformLine(int const lineno)
 		    if (nFirstChar >= screen_columns
 			&& SP->_el_cost <= SP->_el1_cost) {
 			GoTo(lineno, 0);
-			UpdateAttrs(AttrOf(blank));
+			UpdateAttrs(blank);
 			TPUTS_TRACE("clr_eol");
 			putp(clr_eol);
 		    } else {
 			GoTo(lineno, nFirstChar - 1);
-			UpdateAttrs(AttrOf(blank));
+			UpdateAttrs(blank);
 			TPUTS_TRACE("clr_bol");
 			putp(clr_bol);
 		    }
@@ -1214,8 +1281,10 @@ TransformLine(int const lineno)
 		firstChar++;
 	}
 	/* if there wasn't one, we're done */
-	if (firstChar >= screen_columns)
+	if (firstChar >= screen_columns) {
+	    TR(TRACE_UPDATE, (T_RETURN("")));
 	    return;
+	}
 
 	blank = newLine[screen_columns - 1];
 
@@ -1234,6 +1303,7 @@ TransformLine(int const lineno)
 		       newLine + firstChar,
 		       (nLastChar - firstChar + 1) * sizeof(NCURSES_CH_T));
 	    }
+	    TR(TRACE_UPDATE, (T_RETURN("")));
 	    return;
 	}
 
@@ -1273,7 +1343,7 @@ TransformLine(int const lineno)
 	    /* can be -1 if no characters differ */
 	    while (CharEq(newLine[nLastChar], oldLine[oLastChar])) {
 		/* don't split a wide char */
-		if (isnac(newLine[nLastChar]) &&
+		if (isWidecExt(newLine[nLastChar]) &&
 		    !CharEq(newLine[nLastChar - 1], oldLine[oLastChar - 1]))
 		    break;
 		nLastChar--;
@@ -1290,9 +1360,15 @@ TransformLine(int const lineno)
 
 	    if (oLastChar < nLastChar) {
 		int m = max(nLastNonblank, oLastNonblank);
+#if USE_WIDEC_SUPPORT
+		while (isWidecExt(newLine[n + 1]) && n) {
+		    --n;
+		    --oLastChar;
+		}
+#endif
 		GoTo(lineno, n + 1);
-		if (InsCharCost(nLastChar - oLastChar)
-		    > (m - n)) {
+		if ((nLastChar < nLastNonblank)
+		    || InsCharCost(nLastChar - oLastChar) > (m - n)) {
 		    PutRange(oldLine, newLine, lineno, n + 1, m);
 		} else {
 		    InsStr(&newLine[n + 1], nLastChar - oLastChar);
@@ -1314,7 +1390,7 @@ TransformLine(int const lineno)
 		     * setting the video attributes from
 		     * the last character on the row.
 		     */
-		    UpdateAttrs(AttrOf(blank));
+		    UpdateAttrs(blank);
 		    DelChar(oLastChar - nLastChar);
 		}
 	    }
@@ -1326,6 +1402,8 @@ TransformLine(int const lineno)
 	memcpy(oldLine + firstChar,
 	       newLine + firstChar,
 	       (screen_columns - firstChar) * sizeof(NCURSES_CH_T));
+    TR(TRACE_UPDATE, (T_RETURN("")));
+    return;
 }
 
 /*
@@ -1346,7 +1424,7 @@ ClearScreen(NCURSES_CH_T blank)
 #if NCURSES_EXT_FUNCS
     if (SP->_coloron
 	&& !SP->_default_color) {
-	_nc_do_color((int) COLOR_PAIR(SP->_current_attr), 0, FALSE, _nc_outch);
+	_nc_do_color(GET_SCREEN_PAIR(SP), 0, FALSE, _nc_outch);
 	if (!back_color_erase) {
 	    fast_clear = FALSE;
 	}
@@ -1355,7 +1433,7 @@ ClearScreen(NCURSES_CH_T blank)
 
     if (fast_clear) {
 	if (clear_screen) {
-	    UpdateAttrs(AttrOf(blank));
+	    UpdateAttrs(blank);
 	    TPUTS_TRACE("clear_screen");
 	    putp(clear_screen);
 	    SP->_cursrow = SP->_curscol = 0;
@@ -1364,13 +1442,13 @@ ClearScreen(NCURSES_CH_T blank)
 	    SP->_cursrow = SP->_curscol = -1;
 	    GoTo(0, 0);
 
-	    UpdateAttrs(AttrOf(blank));
+	    UpdateAttrs(blank);
 	    TPUTS_TRACE("clr_eos");
 	    tputs(clr_eos, screen_lines, _nc_outch);
 	} else if (clr_eol) {
 	    SP->_cursrow = SP->_curscol = -1;
 
-	    UpdateAttrs(AttrOf(blank));
+	    UpdateAttrs(blank);
 	    for (i = 0; i < screen_lines; i++) {
 		GoTo(i, 0);
 		TPUTS_TRACE("clr_eol");
@@ -1379,7 +1457,7 @@ ClearScreen(NCURSES_CH_T blank)
 	    GoTo(0, 0);
 	}
     } else {
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	for (i = 0; i < screen_lines; i++) {
 	    GoTo(i, 0);
 	    for (j = 0; j < screen_columns; j++)
@@ -1413,7 +1491,7 @@ InsStr(NCURSES_CH_T * line, int count)
     /* The order must match that of InsCharCost. */
     if (parm_ich) {
 	TPUTS_TRACE("parm_ich");
-	tputs(tparm(parm_ich, count), count, _nc_outch);
+	tputs(TPARM_1(parm_ich, count), count, _nc_outch);
 	while (count) {
 	    PutAttrChar(CHREF(*line));
 	    line++;
@@ -1461,12 +1539,14 @@ DelChar(int count)
 {
     int n;
 
-    TR(TRACE_UPDATE, ("DelChar(%d) called, position = (%d,%d)", count,
-		      newscr->_cury, newscr->_curx));
+    TR(TRACE_UPDATE, ("DelChar(%d) called, position = (%ld,%ld)",
+		      count,
+		      (long) newscr->_cury,
+		      (long) newscr->_curx));
 
     if (parm_dch) {
 	TPUTS_TRACE("parm_dch");
-	tputs(tparm(parm_dch, count), count, _nc_outch);
+	tputs(TPARM_1(parm_dch, count), count, _nc_outch);
     } else {
 	for (n = 0; n < count; n++) {
 	    TPUTS_TRACE("delete_character");
@@ -1515,34 +1595,34 @@ scroll_csr_forward(int n, int top, int bot, int miny, int maxy, NCURSES_CH_T bla
 
     if (n == 1 && scroll_forward && top == miny && bot == maxy) {
 	GoTo(bot, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	TPUTS_TRACE("scroll_forward");
 	putp(scroll_forward);
     } else if (n == 1 && delete_line && bot == maxy) {
 	GoTo(top, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	TPUTS_TRACE("delete_line");
 	putp(delete_line);
     } else if (parm_index && top == miny && bot == maxy) {
 	GoTo(bot, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	TPUTS_TRACE("parm_index");
-	tputs(tparm(parm_index, n, 0), n, _nc_outch);
+	tputs(TPARM_2(parm_index, n, 0), n, _nc_outch);
     } else if (parm_delete_line && bot == maxy) {
 	GoTo(top, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	TPUTS_TRACE("parm_delete_line");
-	tputs(tparm(parm_delete_line, n, 0), n, _nc_outch);
+	tputs(TPARM_2(parm_delete_line, n, 0), n, _nc_outch);
     } else if (scroll_forward && top == miny && bot == maxy) {
 	GoTo(bot, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	for (i = 0; i < n; i++) {
 	    TPUTS_TRACE("scroll_forward");
 	    putp(scroll_forward);
 	}
     } else if (delete_line && bot == maxy) {
 	GoTo(top, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	for (i = 0; i < n; i++) {
 	    TPUTS_TRACE("delete_line");
 	    putp(delete_line);
@@ -1573,34 +1653,34 @@ scroll_csr_backward(int n, int top, int bot, int miny, int maxy,
 
     if (n == 1 && scroll_reverse && top == miny && bot == maxy) {
 	GoTo(top, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	TPUTS_TRACE("scroll_reverse");
 	putp(scroll_reverse);
     } else if (n == 1 && insert_line && bot == maxy) {
 	GoTo(top, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	TPUTS_TRACE("insert_line");
 	putp(insert_line);
     } else if (parm_rindex && top == miny && bot == maxy) {
 	GoTo(top, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	TPUTS_TRACE("parm_rindex");
-	tputs(tparm(parm_rindex, n, 0), n, _nc_outch);
+	tputs(TPARM_2(parm_rindex, n, 0), n, _nc_outch);
     } else if (parm_insert_line && bot == maxy) {
 	GoTo(top, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	TPUTS_TRACE("parm_insert_line");
-	tputs(tparm(parm_insert_line, n, 0), n, _nc_outch);
+	tputs(TPARM_2(parm_insert_line, n, 0), n, _nc_outch);
     } else if (scroll_reverse && top == miny && bot == maxy) {
 	GoTo(top, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	for (i = 0; i < n; i++) {
 	    TPUTS_TRACE("scroll_reverse");
 	    putp(scroll_reverse);
 	}
     } else if (insert_line && bot == maxy) {
 	GoTo(top, 0);
-	UpdateAttrs(AttrOf(blank));
+	UpdateAttrs(blank);
 	for (i = 0; i < n; i++) {
 	    TPUTS_TRACE("insert_line");
 	    putp(insert_line);
@@ -1632,13 +1712,13 @@ scroll_idl(int n, int del, int ins, NCURSES_CH_T blank)
 	return ERR;
 
     GoTo(del, 0);
-    UpdateAttrs(AttrOf(blank));
+    UpdateAttrs(blank);
     if (n == 1 && delete_line) {
 	TPUTS_TRACE("delete_line");
 	putp(delete_line);
     } else if (parm_delete_line) {
 	TPUTS_TRACE("parm_delete_line");
-	tputs(tparm(parm_delete_line, n, 0), n, _nc_outch);
+	tputs(TPARM_2(parm_delete_line, n, 0), n, _nc_outch);
     } else {			/* if (delete_line) */
 	for (i = 0; i < n; i++) {
 	    TPUTS_TRACE("delete_line");
@@ -1647,13 +1727,13 @@ scroll_idl(int n, int del, int ins, NCURSES_CH_T blank)
     }
 
     GoTo(ins, 0);
-    UpdateAttrs(AttrOf(blank));
+    UpdateAttrs(blank);
     if (n == 1 && insert_line) {
 	TPUTS_TRACE("insert_line");
 	putp(insert_line);
     } else if (parm_insert_line) {
 	TPUTS_TRACE("parm_insert_line");
-	tputs(tparm(parm_insert_line, n, 0), n, _nc_outch);
+	tputs(TPARM_2(parm_insert_line, n, 0), n, _nc_outch);
     } else {			/* if (insert_line) */
 	for (i = 0; i < n; i++) {
 	    TPUTS_TRACE("insert_line");
@@ -1708,7 +1788,7 @@ _nc_scrolln(int n, int top, int bot, int maxy)
 		putp(save_cursor);
 	    }
 	    TPUTS_TRACE("change_scroll_region");
-	    putp(tparm(change_scroll_region, top, bot));
+	    putp(TPARM_2(change_scroll_region, top, bot));
 	    if (cursor_saved) {
 		TPUTS_TRACE("restore_cursor");
 		putp(restore_cursor);
@@ -1719,7 +1799,7 @@ _nc_scrolln(int n, int top, int bot, int maxy)
 	    res = scroll_csr_forward(n, top, bot, top, bot, blank);
 
 	    TPUTS_TRACE("change_scroll_region");
-	    putp(tparm(change_scroll_region, 0, maxy));
+	    putp(TPARM_2(change_scroll_region, 0, maxy));
 	    SP->_cursrow = SP->_curscol = -1;
 	}
 
@@ -1731,7 +1811,7 @@ _nc_scrolln(int n, int top, int bot, int maxy)
 	 */
 	if (res != ERR
 	    && (non_dest_scroll_region || (memory_below && bot == maxy))) {
-	    NCURSES_CH_T blank2 = NewChar(BLANK_TEXT);
+	    static const NCURSES_CH_T blank2 = NewChar(BLANK_TEXT);
 	    if (bot == maxy && clr_eos) {
 		GoTo(bot - n + 1, 0);
 		ClrToEOS(blank2);
@@ -1754,7 +1834,7 @@ _nc_scrolln(int n, int top, int bot, int maxy)
 		putp(save_cursor);
 	    }
 	    TPUTS_TRACE("change_scroll_region");
-	    putp(tparm(change_scroll_region, top, bot));
+	    putp(TPARM_2(change_scroll_region, top, bot));
 	    if (cursor_saved) {
 		TPUTS_TRACE("restore_cursor");
 		putp(restore_cursor);
@@ -1765,7 +1845,7 @@ _nc_scrolln(int n, int top, int bot, int maxy)
 	    res = scroll_csr_backward(-n, top, bot, top, bot, blank);
 
 	    TPUTS_TRACE("change_scroll_region");
-	    putp(tparm(change_scroll_region, 0, maxy));
+	    putp(TPARM_2(change_scroll_region, 0, maxy));
 	    SP->_cursrow = SP->_curscol = -1;
 	}
 
@@ -1777,7 +1857,7 @@ _nc_scrolln(int n, int top, int bot, int maxy)
 	 */
 	if (res != ERR
 	    && (non_dest_scroll_region || (memory_above && top == 0))) {
-	    NCURSES_CH_T blank2 = NewChar(BLANK_TEXT);
+	    static const NCURSES_CH_T blank2 = NewChar(BLANK_TEXT);
 	    for (i = 0; i < -n; i++) {
 		GoTo(i + top, 0);
 		ClrToEOL(blank2, FALSE);
@@ -1800,7 +1880,7 @@ NCURSES_EXPORT(void)
 _nc_screen_resume(void)
 {
     /* make sure terminal is in a sane known state */
-    SP->_current_attr = A_NORMAL;
+    SetAttr(SCREEN_ATTRS(SP), A_NORMAL);
     newscr->_clear = TRUE;
 
     /* reset color pairs and definitions */
@@ -1848,11 +1928,11 @@ _nc_screen_init(void)
 NCURSES_EXPORT(void)
 _nc_screen_wrap(void)
 {
-    UpdateAttrs(A_NORMAL);
+    UpdateAttrs(normal);
 #if NCURSES_EXT_FUNCS
     if (SP->_coloron
 	&& !SP->_default_color) {
-	NCURSES_CH_T blank = NewChar(BLANK_TEXT);
+	static const NCURSES_CH_T blank = NewChar(BLANK_TEXT);
 	SP->_default_color = TRUE;
 	_nc_do_color(-1, 0, FALSE, _nc_outch);
 	SP->_default_color = FALSE;
@@ -1871,7 +1951,7 @@ _nc_screen_wrap(void)
 NCURSES_EXPORT(void)
 _nc_do_xmc_glitch(attr_t previous)
 {
-    attr_t chg = XMC_CHANGES(previous ^ SP->_current_attr);
+    attr_t chg = XMC_CHANGES(previous ^ AttrOf(SCREEN_ATTRS(SP)));
 
     while (chg != 0) {
 	if (chg & 1) {
