@@ -1,3 +1,30 @@
+/****************************************************************************
+ * Copyright (c) 1998-2005,2006 Free Software Foundation, Inc.              *
+ *                                                                          *
+ * Permission is hereby granted, free of charge, to any person obtaining a  *
+ * copy of this software and associated documentation files (the            *
+ * "Software"), to deal in the Software without restriction, including      *
+ * without limitation the rights to use, copy, modify, merge, publish,      *
+ * distribute, distribute with modifications, sublicense, and/or sell       *
+ * copies of the Software, and to permit persons to whom the Software is    *
+ * furnished to do so, subject to the following conditions:                 *
+ *                                                                          *
+ * The above copyright notice and this permission notice shall be included  *
+ * in all copies or substantial portions of the Software.                   *
+ *                                                                          *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
+ * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
+ * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
+ * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
+ * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
+ *                                                                          *
+ * Except as contained in this notice, the name(s) of the above copyright   *
+ * holders shall not be used in advertising or otherwise to promote the     *
+ * sale, use or other dealings in this Software without prior written       *
+ * authorization.                                                           *
+ ****************************************************************************/
 /* 
  * bs.c - original author: Bruce Holloway
  *		salvo option by: Chuck A DeGaul
@@ -7,13 +34,13 @@
  * v2.0 featuring strict ANSI/POSIX conformance, November 1993.
  * v2.1 with ncurses mouse support, September 1995
  *
- * $Id: bs.c,v 1.39 2003/12/06 18:10:13 tom Exp $
+ * $Id: bs.c,v 1.44 2006/05/20 15:38:52 tom Exp $
  */
 
-#include <time.h>
-
 #include <test.priv.h>
 
+#include <time.h>
+
 #ifndef SIGIOT
 #define SIGIOT SIGABRT
 #endif
@@ -149,7 +176,8 @@ static int salvo, blitz, closepack;
 
 static RETSIGTYPE uninitgame(int sig) GCC_NORETURN;
 
-static RETSIGTYPE uninitgame(int sig GCC_UNUSED)
+static RETSIGTYPE
+uninitgame(int sig GCC_UNUSED)
 /* end the game, either normally or due to signal */
 {
     clear();
@@ -190,11 +218,7 @@ intro(void)
 
     srand((unsigned) (time(0L) + getpid()));	/* Kick the random number generator */
 
-    (void) signal(SIGINT, uninitgame);
-    (void) signal(SIGINT, uninitgame);
-    (void) signal(SIGIOT, uninitgame);	/* for assert(3) */
-    if (signal(SIGQUIT, SIG_IGN) != SIG_IGN)
-	(void) signal(SIGQUIT, uninitgame);
+    CATCHALL(uninitgame);
 
     if ((tmpname = getlogin()) != 0) {
 	(void) strcpy(name, tmpname);
@@ -249,7 +273,7 @@ intro(void)
 
 /* VARARGS1 */
 static void
-prompt(int n, const char *f, const char *s)
+prompt(int n, NCURSES_CONST char *f, const char *s)
 /* print a message at the prompt line */
 {
     (void) move(PROMPTLINE + n, 0);
@@ -839,7 +863,6 @@ plyturn(void)
 	}
 	(void) printw(m, ss->name);
 	(void) beep();
-	return (awinna() == -1);
     }
     return (hit);
 }
@@ -949,13 +972,17 @@ cpufire(int x, int y)
     attrset(0);
 #endif /* A_COLOR */
 
-    return ((hit ? (sunk ? S_SUNK : S_HIT) : S_MISS) ? TRUE : FALSE);
+    return hit ? (sunk ? S_SUNK : S_HIT) : S_MISS;
 }
 
 /*
  * This code implements a fairly irregular FSM, so please forgive the rampant
  * unstructuredness below. The five labels are states which need to be held
  * between computer turns.
+ *
+ * The FSM is not externally reset to RANDOM_FIRE if the player wins. Instead,
+ * the other states check for "impossible" conditions which signify a new
+ * game, then if found transition to RANDOM_FIRE.
  */
 static bool
 cputurn(void)
@@ -1003,13 +1030,17 @@ cputurn(void)
 	if (navail == 0)	/* no valid places for shots adjacent... */
 	    goto refire;	/* ...so we must random-fire */
 	else {
-	    for (d = 0, n = rnd(navail) + 1; n; n--)
-		while (used[d])
-		    d++;
+	    n = rnd(navail) + 1;
+	    for (d = 0; used[d]; d++) ;
+	    /* used[d] is first that == 0 */
+	    for (; n > 1; n--)
+		while (used[++d]) ;
+	    /* used[d] is next that == 0 */
 
-	    assert(d <= 4);
+	    assert(d < 4);
+	    assert(used[d] == FALSE);
 
-	    used[d] = FALSE;
+	    used[d] = TRUE;
 	    x = ts.x + xincr[d * 2];
 	    y = ts.y + yincr[d * 2];
 
@@ -1040,7 +1071,7 @@ cputurn(void)
 	break;
 
     case REVERSE_JUMP:		/* nail down the ship's other end */
-	d = ts.dir + 4;
+	d = (ts.dir + 4) % 8;
 	x = ts.x + ts.hits * xincr[d];
 	y = ts.y + ts.hits * yincr[d];
 	if (POSSIBLE(x, y) && (hit = cpufire(x, y))) {
@@ -1053,7 +1084,7 @@ cputurn(void)
 	    next = RANDOM_FIRE;
 	break;
 
-    case SECOND_PASS:		/* kill squares not caught on first pass */
+    case SECOND_PASS:		/* continue shooting after reversing */
 	x = ts.x + xincr[ts.dir];
 	y = ts.y + yincr[ts.dir];
 	if (POSSIBLE(x, y) && (hit = cpufire(x, y))) {
@@ -1067,14 +1098,11 @@ cputurn(void)
 	break;
     }
 
-    /* check for continuation and/or winner */
+    /* pause between shots in salvo */
     if (salvo) {
 	(void) refresh();
 	(void) sleep(1);
     }
-    if (awinna() != -1)
-	return (FALSE);
-
 #ifdef DEBUG
     (void) mvprintw(PROMPTLINE + 2, 0,
 		    "New state %d, x=%d, y=%d, d=%d",
@@ -1213,7 +1241,7 @@ main(int argc, char *argv[])
 		    }
 		}
 	    } else
-		while (turn ? cputurn() : plyturn())
+		while ((turn ? cputurn() : plyturn()) && awinna() == -1)
 		    continue;
 	    turn = OTHER;
 	}
