@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2006,2007 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -27,9 +27,9 @@
  ****************************************************************************/
 
 /****************************************************************************
- *  Author: Thomas E. Dickey <dickey@clark.net> 1996                        *
+ *  Author: Thomas E. Dickey                    1996-on                     *
  ****************************************************************************/
-/* $Id: test.priv.h,v 1.46 2003/10/19 00:04:24 tom Exp $ */
+/* $Id: test.priv.h,v 1.68 2007/06/30 17:53:09 tom Exp $ */
 
 #ifndef __TEST_PRIV_H
 #define __TEST_PRIV_H 1
@@ -41,28 +41,52 @@
  * with wide-character ncurses.
  */
 #ifdef  HAVE_LIBFORMW
-#define HAVE_LIBFORM 1
+#define HAVE_LIBFORMW 1
 #endif
 
 #ifdef  HAVE_LIBMENUW
-#define HAVE_LIBMENU 1
+#define HAVE_LIBMENUW 1
 #endif
 
 #ifdef  HAVE_LIBPANELW
-#define HAVE_LIBPANEL 1
+#define HAVE_LIBPANELW 1
 #endif
 
 /*
- * Fallback definitions to accommodate broken compilers
+ * Fallback definitions to accommodate broken compilers.
  */
 #ifndef HAVE_CURSES_VERSION
 #define HAVE_CURSES_VERSION 0
 #endif
 
+#ifndef HAVE_FILTER
+#define HAVE_FILTER 0
+#endif
+
 #ifndef HAVE_FORM_H
 #define HAVE_FORM_H 0
 #endif
 
+#ifndef HAVE_GETBEGX
+#define HAVE_GETBEGX 0
+#endif
+
+#ifndef HAVE_GETCURX
+#define HAVE_GETCURX 0
+#endif
+
+#ifndef HAVE_GETMAXX
+#define HAVE_GETMAXX 0
+#endif
+
+#ifndef HAVE_GETOPT_H
+#define HAVE_GETOPT_H 0
+#endif
+
+#ifndef HAVE_GETWIN
+#define HAVE_GETWIN 0
+#endif
+
 #ifndef HAVE_LIBFORM
 #define HAVE_LIBFORM 0
 #endif
@@ -83,6 +107,14 @@
 #define HAVE_MENU_H 0
 #endif
 
+#ifndef HAVE_MVVLINE
+#define HAVE_MVVLINE 0
+#endif
+
+#ifndef HAVE_MVWVLINE
+#define HAVE_MVWVLINE 0
+#endif
+
 #ifndef HAVE_NAPMS
 #define HAVE_NAPMS 1
 #endif
@@ -95,10 +127,62 @@
 #define HAVE_PANEL_H 0
 #endif
 
+#ifndef HAVE_PUTWIN
+#define HAVE_PUTWIN 0
+#endif
+
+#ifndef HAVE_RESIZE_TERM
+#define HAVE_RESIZE_TERM 0
+#endif
+
+#ifndef HAVE_RIPOFFLINE
+#define HAVE_RIPOFFLINE 0
+#endif
+
+#ifndef HAVE_SETUPTERM
+#define HAVE_SETUPTERM 0
+#endif
+
 #ifndef HAVE_SLK_COLOR
 #define HAVE_SLK_COLOR 0
 #endif
 
+#ifndef HAVE_SLK_INIT
+#define HAVE_SLK_INIT 0
+#endif
+
+#ifndef HAVE_TERMATTRS
+#define HAVE_TERMATTRS 0
+#endif
+
+#ifndef HAVE_TERMNAME
+#define HAVE_TERMNAME 0
+#endif
+
+#ifndef HAVE_TGETENT
+#define HAVE_TGETENT 0
+#endif
+
+#ifndef HAVE_TIGETNUM
+#define HAVE_TIGETNUM 0
+#endif
+
+#ifndef HAVE_TYPEAHEAD
+#define HAVE_TYPEAHEAD 0
+#endif
+
+#ifndef HAVE_TIGETSTR
+#define HAVE_TIGETSTR 0
+#endif
+
+#ifndef HAVE_WINSSTR
+#define HAVE_WINSSTR 0
+#endif
+
+#ifndef HAVE_USE_DEFAULT_COLORS
+#define HAVE_USE_DEFAULT_COLORS 0
+#endif
+
 #ifndef HAVE_WRESIZE
 #define HAVE_WRESIZE 0
 #endif
@@ -107,14 +191,14 @@
 #define NCURSES_EXT_FUNCS 0
 #endif
 
-#ifndef NCURSES_NOMACROS
-#define NCURSES_NOMACROS 0
-#endif
-
 #ifndef NEED_PTEM_H
 #define NEED_PTEM_H 0
 #endif
 
+#ifndef NO_LEAKS
+#define NO_LEAKS 0
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -125,19 +209,35 @@
 
 #include <signal.h>	/* include before curses.h to work around glibc bug */
 
-#if defined(HAVE_NCURSESW_NCURSES_H)
+#if defined(HAVE_XCURSES)
+#include <xcurses.h>
+#elif defined(HAVE_NCURSESW_NCURSES_H)
 #include <ncursesw/curses.h>
-#include <ncursesw/term.h>
 #elif defined(HAVE_NCURSES_NCURSES_H)
 #include <ncurses/curses.h>
-#include <ncurses/term.h>
 #else
 #include <curses.h>
+#endif
+
+#if defined(HAVE_XCURSES)
+/* no other headers */
+#undef  HAVE_SETUPTERM		/* nonfunctional */
+#define HAVE_SETUPTERM 0
+#undef  HAVE_TGETENT		/* nonfunctional */
+#define HAVE_TGETENT 0
+#undef  HAVE_TIGETSTR		/* nonfunctional */
+#define HAVE_TIGETSTR 0
+#elif defined(HAVE_NCURSESW_TERM_H)
+#include <ncursesw/term.h>
+#elif defined(HAVE_NCURSES_TERM_H)
+#include <ncurses/term.h>
+#elif defined(HAVE_TERM_H)
 #include <term.h>
 #endif
 
-#if NCURSES_NOMACROS
-#include <nomacros.h>
+#ifdef NCURSES_VERSION
+#define HAVE_COLOR_SET 1
+#define HAVE_CHGAT 1
 #endif
 
 #if HAVE_GETOPT_H
@@ -162,6 +262,9 @@ extern int optind;
 #ifndef GCC_NORETURN
 #define GCC_NORETURN /* nothing */
 #endif
+#ifndef GCC_PRINTFLIKE
+#define GCC_PRINTFLIKE(a,b) /* nothing */
+#endif
 #ifndef GCC_UNUSED
 #define GCC_UNUSED /* nothing */
 #endif
@@ -170,6 +273,14 @@ extern int optind;
 #define getnstr(s,n) getstr(s)
 #endif
 
+#ifndef USE_SOFTKEYS
+#if HAVE_SLK_INIT
+#define USE_SOFTKEYS 1
+#else
+#define USE_SOFTKEYS 0
+#endif
+#endif
+
 #ifndef USE_WIDEC_SUPPORT
 #if defined(_XOPEN_SOURCE_EXTENDED) && defined(WACS_ULCORNER)
 #define USE_WIDEC_SUPPORT 1
@@ -198,7 +309,7 @@ extern int optind;
 
 #ifndef HAVE_TYPE_ATTR_T
 #if !USE_WIDEC_SUPPORT
-#define attr_t long
+#define attr_t chtype
 #endif
 #endif
 
@@ -209,6 +320,10 @@ extern int optind;
 #define NCURSES_CH_T cchar_t
 #endif
 
+#ifndef NCURSES_OPAQUE
+#define NCURSES_OPAQUE 0
+#endif
+
 #ifndef CCHARW_MAX
 #define CCHARW_MAX 5
 #endif
@@ -224,21 +339,63 @@ extern int optind;
 #define KEY_MIN 256	/* not defined in Solaris 8 */
 #endif
 
-#ifndef getcurx
+/*
+ * Workaround for HPUX
+ */
+#if defined(__hpux) && !defined(NCURSES_VERSION)
+#define getbegx(w) __getbegx(w)
+#define getbegy(w) __getbegy(w)
+#define getcurx(w) __getcurx(w)
+#define getcury(w) __getcury(w)
+#define getmaxx(w) __getmaxx(w)
+#define getmaxy(w) __getmaxy(w)
+#define getparx(w) __getparx(w)
+#define getpary(w) __getpary(w)
+#endif
+
+/*
+ * These usually are implemented as macros, but may be functions.
+ */
+#if !defined(getcurx) && !HAVE_GETCURX
 #define getcurx(win)            ((win)?(win)->_curx:ERR)
 #define getcury(win)            ((win)?(win)->_cury:ERR)
 #endif
 
-#ifndef getbegx
+#if !defined(getbegx) && !HAVE_GETBEGX
 #define getbegx(win)            ((win)?(win)->_begx:ERR)
 #define getbegy(win)            ((win)?(win)->_begy:ERR)
 #endif
 
-#ifndef getmaxx
+#if !defined(getmaxx) && !HAVE_GETMAXX
 #define getmaxx(win)            ((win)?((win)->_maxx + 1):ERR)
 #define getmaxy(win)            ((win)?((win)->_maxy + 1):ERR)
 #endif
 
+#if !defined(mvwvline) && !HAVE_MVWVLINE
+#define mvwvline(w,y,x,ch,n)    (move(y,x) == ERR ? ERR : wvline(w,ch,n))
+#define mvwhline(w,y,x,ch,n)    (move(y,x) == ERR ? ERR : whline(w,ch,n))
+#endif
+
+#if !defined(mvvline) && !HAVE_MVVLINE
+#define mvvline(y,x,ch,n)       (move(y,x) == ERR ? ERR : vline(ch,n))
+#define mvhline(y,x,ch,n)       (move(y,x) == ERR ? ERR : hline(ch,n))
+#endif
+
+/*
+ * Try to accommodate curses implementations that have no terminfo support.
+ */
+#if HAVE_TIGETNUM
+#define TIGETNUM(ti,tc) tigetnum(ti)
+#else
+#define TIGETNUM(ti,tc) tgetnum(tc)
+#endif
+
+#if HAVE_TIGETSTR
+#define TIGETSTR(ti,tc) tigetstr(ti)
+#else
+#define TIGETSTR(ti,tc) tgetstr(tc,&area_pointer)
+#endif
+
 /* ncurses implements tparm() with varargs, X/Open with a fixed-parameter list
  * (which is incompatible with legacy usage, doesn't solve any problems).
  */
@@ -251,6 +408,10 @@ extern int optind;
 
 #if defined(NCURSES_VERSION) && HAVE_NC_ALLOC_H
 #include <nc_alloc.h>
+#if HAVE_NC_FREEALL && defined(USE_TINFO)
+#undef ExitProgram
+#define ExitProgram(code) _nc_free_tinfo(code)
+#endif
 #else
 #define typeMalloc(type,n) (type *) malloc((n) * sizeof(type))
 #define typeRealloc(type,n,p) (type *) realloc(p, (n) * sizeof(type))
@@ -276,12 +437,12 @@ extern int optind;
 #endif
 
 /* out-of-band values for representing absent capabilities */
-#define ABSENT_BOOLEAN		(-1)		/* 255 */
+#define ABSENT_BOOLEAN		((signed char)-1)	/* 255 */
 #define ABSENT_NUMERIC		(-1)
 #define ABSENT_STRING		(char *)0
 
 /* out-of-band values for representing cancels */
-#define CANCELLED_BOOLEAN	(char)(-2)	/* 254 */
+#define CANCELLED_BOOLEAN	((signed char)-2)	/* 254 */
 #define CANCELLED_NUMERIC	(-2)
 #define CANCELLED_STRING	(char *)(-1)
 
@@ -291,4 +452,11 @@ extern int optind;
 
 #define VT_ACSC "``aaffggiijjkkllmmnnooppqqrrssttuuvvwwxxyyzz{{||}}~~"
 
+#define CATCHALL(handler) { \
+		int nsig; \
+		for (nsig = SIGHUP; nsig < SIGTERM; ++nsig) \
+		    if (nsig != SIGKILL) \
+			signal(nsig, handler); \
+	    }
+
 #endif /* __TEST_PRIV_H */
